# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from re import compile, escape

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import (
    ComicControlScraper,
    WordPressNavi,
    WordPressScraper,
    WordPressWebcomic,
)


class Namesake(ComicControlScraper):
    url = 'http://namesakecomic.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'the-journey-begins'


class NatalieDee(_BasicScraper):
    url = 'http://www.nataliedee.com/'
    rurl = escape(url)
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '022806'
    imageSearch = compile(tagre("img", "src", r'(%s\d+/[^"]+)' % rurl,
                                before="overflow"))
    prevSearch = compile(tagre("a", "href", r'([^"]+)') + "&lt;&lt; Yesterday")
    help = 'Index format: mmddyy'
    namer = joinPathPartsNamer(imageparts=range(-2, 0), joinchar='-')


class Nedroid(WordPressScraper):
    url = 'http://nedroid.com/'
    prevSearch = '//a[@rel="prev"]'


class NeoCTC(_ParserScraper):
    url = 'http://www.hirezfox.com/neoctc/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '20071205'
    imageSearch = '//img[contains(@src, "neoctc/comics")]'
    prevSearch = '//a[./img[@alt="Previous Day"]]'
    multipleImagesPerStrip = True


class NeoEarth(_BasicScraper):
    url = 'http://www.neo-earth.com/NE/'
    stripUrl = url + 'index.php?date=%s'
    firstStripUrl = stripUrl % '2007-03-23'
    imageSearch = compile(r'<img src="(strips/.+?)"')
    prevSearch = compile(r'<a href="(.+?)">Previous</a>')
    help = 'Index format: yyyy-mm-dd'


class NerfNow(WordPressScraper):
    url = 'https://www.nerfnow.com/'
    prevSearch = '//li[@id="nav_previous"]/a'


class Newshounds(_ParserScraper):
    stripUrl = 'http://www.newshounds.com/%s.html'
    url = stripUrl % 'nh2/20140929'
    firstStripUrl = stripUrl % 'nh1/19971101'
    imageSearch = '//img[@class="ksc"]'
    prevSearch = '//a[./img[@alt="Previous comic"]]'
    endOfLife = True

    def getPrevUrl(self, url, data):
        # Add navigation link between comic and graphic novel
        if url == self.stripUrl % 'nh2/20070201':
            return self.stripUrl % 'nh1/20061208'
        return super(Newshounds, self).getPrevUrl(url, data)


class NewWorld(WordPressScraper):
    url = ('https://web.archive.org/web/20190718012133/'
        'http://www.tfsnewworld.com/')
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2007/08/30/63'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True
    help = 'Index format: yyyy/mm/dd/stripn'


class NeverSatisfied(ComicControlScraper):
    url = 'https://www.neversatisfiedcomic.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'never-satisfied'


class NichtLustig(_BasicScraper):
    url = 'https://joscha.com/'
    starter = bounceStarter
    stripUrl = url + 'nichtlustig/%s/'
    firstStripUrl = stripUrl % '000501'
    lang = 'de'
    imageSearch = compile(tagre("img", "src", r'(https://joscha.com/data/media/cartoons/[0-9a-f-_]+.png)'))
    prevSearch = compile(tagre("a", "href", r'(https://joscha.com/nichtlustig/\d+/)', after="next"))
    nextSearch = compile(tagre("a", "href", r'(https://joscha.com/nichtlustig/\d+/)', after="prev"))
    namer = joinPathPartsNamer(pageparts=(-1,))
    help = 'Index format: yymmdd'


class Nicky510(WordPressNavi):
    url = ('https://web.archive.org/web/20160510215718/'
        'http://www.nickyitis.com/')
    endOfLife = True


class Nightshift(ParserScraper):
    url = 'https://poecatcomix.com/nightshiftcomic/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % 'ns-issue-1-cover'
    imageSearch = '//img[@class="scale-with-grid wp-post-image"]'
    prevSearch = '//a[d:class("fixed-nav-prev")]'
    latestSearch = '//div[@class="post-title"]//a'
    starter = indirectStarter
    adult = True
    namer = joinPathPartsNamer(pageparts=(-2,))


class Nimona(_ParserScraper):
    url = ('https://web.archive.org/web/20141008095502/'
        'http://gingerhaze.com/nimona/')
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % "page-1"
    imageSearch = '//div[d:class("field-name-field-comic-page")]//img'
    prevSearch = '//a[img[contains(@src, "/comicdrop_prev_label")]]'
    endOfLife = True


class NineToNine(_ParserScraper):
    url = 'https://www.tigerknight.com/99'
    stripUrl = url + '/%s'
    firstStripUrl = stripUrl % '2014-01-01'
    imageSearch = '//img[d:class("comic-image")]'
    prevSearch = '//a[./span[contains(text(), "Previous")]]'
    multipleImagesPerStrip = True


class NobodyScores(_BasicScraper):
    url = 'http://nobodyscores.loosenutstudio.com/'
    rurl = escape(url)
    stripUrl = url + 'index.php?id=%s'
    firstStripUrl = stripUrl % '4'
    imageSearch = compile(tagre("img", "src", r'(%scomix/[^"]+)' % rurl))
    multipleImagesPerStrip = True
    prevSearch = compile(r'<a href="(%sindex.php.+?)">the one before </a>' % rurl)
    help = 'Index format: nnn'


class NoNeedForBushido(_ParserScraper):
    url = 'http://nn4b.com/'
    stripUrl = url + 'comic/%s'
    imageSearch = '//div[@id="comic-image"]//img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: nnn'


class NonPlayerCharacter(ParserScraper):
    url = 'https://www.lfg.co/'
    stripUrl = url + 'npc/comic/%s/'
    firstStripUrl = stripUrl % '1-1'
    imageSearch = '//div[@id="comic-img"]//img'
    prevSearch = '//a[@class="comic-nav-prev"]'
    latestSearch = '//div[@id="feature-npc-footer"]/a[contains(@href, "npc/comic/")]'
    starter = indirectStarter
    namer = joinPathPartsNamer(pageparts=(-1,))


class NotAVillain(WordPressWebcomic):
    url = 'https://navcomic.com/'
    stripUrl = url + 'not-a-villain/%s/'
    firstStripUrl = stripUrl % 'v1-001'

    def namer(self, image_url, page_url):
        filename = util.urlpathsplit(image_url)[-1]
        # Fix filenames missing "Page"
        if filename[2].isdigit():
            filename = filename[0] + '-Page' + filename[2:]
        # Fix filenames of early comics
        filename = filename.replace('Page-', '1-Page')
        if filename.startswith('0-Page'):
            filename = '1' + filename[1:]
        return filename


class NotInventedHere(_ParserScraper):
    url = 'http://notinventedhe.re/'
    stripUrl = url + 'on/%s'
    firstStripUrl = stripUrl % '2009-9-21'
    imageSearch = '//div[@id="comic-content"]//img'
    prevSearch = '//a[@id="nav-previous"]'
    help = 'Index format: yyyy-m-d'


class Nukees(_BasicScraper):
    url = 'http://www.nukees.com/'
    stripUrl = url + 'd/%s'
    firstStripUrl = stripUrl % '19970121'
    imageSearch = compile(r'"comic".+?"(/comics/.+?)"')
    prevSearch = compile(r'"(/d/.+?)".+?previous')
    help = 'Index format: yyyymmdd.html'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
import os
import re
from operator import attrgetter

import pytest
from xdist.dsession import LoadScopeScheduling

from dosagelib.scraper import scrapers


def get_test_scrapers():
    """Return scrapers that should be tested."""
    if 'TESTALL' in os.environ:
        # test all comics (this will take some time)
        # Ignore known-broken (and never-to-be-fixed) modules
        scraper_pattern = '^(?!Schuelert)'
    elif 'TESTCOMICS' in os.environ:
        scraper_pattern = os.environ['TESTCOMICS']
    else:
        # Get limited number of scraper tests as default
        testscrapernames = [
            # "classic" BasicScraper
            'Deathbulge',
            # complex ParserScraper
            'GoComics/CalvinAndHobbes',
            # WordPressScraper
            'GrrlPower',
        ]
        scraper_pattern = '^(' + '|'.join(testscrapernames) + ')$'

    matcher = re.compile(scraper_pattern)
    return [
        scraperobj for scraperobj in scrapers.all()
        if matcher.match(scraperobj.name)
    ]


def pytest_generate_tests(metafunc):
    if 'scraperobj' in metafunc.fixturenames:
        scrapers = sorted(get_test_scrapers(), key=attrgetter('name'))
        metafunc.parametrize('scraperobj', scrapers, ids=attrgetter('name'))


class LoadModScheduling(LoadScopeScheduling):
    """Implement load scheduling for comic modules. See xdist for details."""

    def _split_scope(self, nodeid):
        mod, test = nodeid.split("::", 1)
        return mod + "::" + test.split("/", 1)[0]


@pytest.hookimpl(trylast=True)
def pytest_xdist_make_scheduler(config, log):
    return LoadModScheduling(config, log)



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from ..scraper import ParserScraper
from ..helpers import indirectStarter


class Wrongside(ParserScraper):
    baseUrl = 'http://ayzewi.com/maingallery3/'
    url = baseUrl + 'index.php?/category/5'
    stripUrl = baseUrl + 'picture.php?%s'
    firstStripUrl = stripUrl % '/175/category/21'
    imageSearch = '//img[@id="theMainImage"]/@src'
    prevSearch = '//a[contains(@title, "Previous :")]'

    def starter(self):
        archivePage = self.getPage(self.url)
        chapterUrls = self.match(archivePage, '//ul[d:class("albThumbs")]//a/@href')
        self.archive = []
        for chapterUrl in chapterUrls:
            chapterPage = self.getPage(chapterUrl)
            self.archive.append(self.match(chapterPage, '(//ul[@id="thumbnails"]//a/@href)[last()]')[0])
        return self.archive[0]

    def getPrevUrl(self, url, data):
        if self.match(data, self.prevSearch) == [] and len(self.archive) > 0:
            return self.archive.pop()
        return super(Wrongside, self).getPrevUrl(url, data)

    def namer(self, imageUrl, pageUrl):
        page = self.getPage(pageUrl)
        title = self.match(page, '//div[d:class("browsePath")]/h2/text()')[0]
        return title.replace('"', '') + '.' + imageUrl.rsplit('.', 1)[-1]


class WrongsideBeginnings(Wrongside):
    name = 'Wrongside/Beginnings'
    baseUrl = 'http://ayzewi.com/maingallery3/'
    url = baseUrl + 'index.php?/category/4'
    stripUrl = baseUrl + 'picture.php?%s'
    firstStripUrl = stripUrl % '/2/category/18'


class WrongsideSideStories(ParserScraper):
    baseUrl = 'http://ayzewi.com/maingallery3/'
    stripUrl = baseUrl + 'picture.php?%s'
    imageSearch = '//img[@id="theMainImage"]/@src'
    prevSearch = '//a[contains(@title, "Previous :")]'
    latestSearch = '(//ul[@id="thumbnails"]//a/@href)[last()]'
    starter = indirectStarter

    def __init__(self, name, category, first, last=None):
        super().__init__('Wrongside/' + name)
        self.url = self.baseUrl + 'index.php?/category/' + category
        self.firstStripUrl = self.stripUrl % ('/' + first + '/category/' + category)

        if last:
            self.endOfLife = True

    @classmethod
    def getmodules(cls):
        return (
            cls('AnarkisRising', '7', '302'),
            cls('CommonsDreams', '9', '324'),
            cls('Faith', '11', '349'),
            cls('Sarah', '10', '337'),
            cls('ThereAreNoAviansHere', '8', '313'),
            cls('TheScientificProphet', '13', '358'),
            cls('TheStrangers', '12', '361'),
        )

    def namer(self, imageUrl, pageUrl):
        page = self.getPage(pageUrl)
        title = self.match(page, '//div[d:class("browsePath")]/h2/text()')[0]
        return title.replace('"', '') + '.' + imageUrl.rsplit('.', 1)[-1]



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from ..scraper import ParserScraper


class WebToons(ParserScraper):
    imageSearch = '//img[contains(@class, "_images")]/@data-url'
    prevSearch = '//a[contains(@class, "_prevEpisode")]'
    multipleImagesPerStrip = True

    def __init__(self, name: str, path: str, titlenum: int) -> None:
        super().__init__('WebToons/' + name)
        # Since pages on WebToon are composed of many images, we can probably
        # move a lot faster then default
        self.session.add_throttle('webtoon-phinf.pstatic.net', 0.0, 0.05)

        baseUrl = 'https://www.webtoons.com/en/'
        self.url = baseUrl + path + '/episode/viewer?title_no=' + str(titlenum)
        self.listUrl = baseUrl + path + '/list?title_no=' + str(titlenum)
        self.stripUrl = self.url + '&episode_no=%s'
        self.firstStripUrl = self.stripUrl % '1'

    def starter(self) -> str:
        # Avoid age/GDPR gate
        for cookie in ('needGDPR', 'needCCPA', 'needCOPPA'):
            self.session.cookies.set(cookie, 'false', domain='webtoons.com')
        # Find current episode number
        listPage = self.getPage(self.listUrl)
        currentEpisode = self.match(listPage, '//div[d:class("detail_lst")]/ul/li')[0].attrib['data-episode-no']
        # Check for completed tag
        self.endOfLife = not self.match(listPage, '//div[@id="_asideDetail"]//span[d:class("txt_ico_completed2")]')
        return self.stripUrl % currentEpisode

    def extract_image_urls(self, url, data):
        # Save link order for position-based filenames
        self._cached_image_urls = super().extract_image_urls(url, data)
        # Update firstStripUrl with the correct episode title
        if url.rsplit('=', 1)[-1] == '1':
            self.firstStripUrl = url
        return self._cached_image_urls

    def namer(self, imageUrl, pageUrl):
        # Construct filename from episode number and image position on page
        episodeNum = pageUrl.rsplit('=', 1)[-1]
        imageNum = self._cached_image_urls.index(imageUrl)
        imageExt = pageUrl.rsplit('.', 1)[-1].split('?', 1)[0]
        return "%s-%03d.%s" % (episodeNum, imageNum, imageExt)

    @classmethod
    def getmodules(cls):
        return (
            cls('1000', 'action/one-thousand', 1217),
            cls('10thDimensionBoys', 'comedy/10th-dimension-boys', 71),
            cls('1111Animals', 'comedy/1111-animals', 437),
            cls('2015SpaceSeries', 'sf/2015-space-series', 391),
            cls('3SecondStrip', 'comedy/3-second-strip', 380),
            cls('99ReinforcedStick', 'comedy/99-reinforced-wooden-stick', 4286),
            cls('ABittersweetLife', 'slice-of-life/a-bittersweet-life', 294),
            cls('AboutDeath', 'drama/about-death', 82),
            cls('Acception', 'drama/acception', 1513),
            cls('AcesWild', 'challenge/aces-wild', 689025),
            cls('Acursian', 'supernatural/acursian', 1452),
            cls('AdventuresOfGod', 'comedy/adventures-of-god', 853),
            cls('AerialMagic', 'fantasy/aerial-magic', 1358),
            cls('AgeMatters', 'romance/age-matters', 1364),
            cls('AGoodDayToBeADog', 'romance/a-good-day-tobe-a-dog', 1390),
            cls('Aisopos', 'drama/aisopos', 76),
            cls('AliceElise', 'fantasy/alice-elise', 1481),
            cls('AlloyComics', 'canvas/alloy-comics', 747447),
            cls('AllThatWeHopeToBe', 'slice-of-life/all-that-we-hope-to-be', 470),
            cls('AllThatYouAre', 'drama/all-that-you-are', 403),
            cls('AlwaysHuman', 'romance/always-human', 557),
            cls('AMPED', 'challenge/amped', 532009),
            cls('Annarasumanara', 'drama/annarasumanara', 77),
            cls('AphroditeIX', 'sf/aphroditeix', 1451),
            cls('ApocalypticHorseplay', 'supernatural/apocalyptic-horseplay', 635),
            cls('AprilFlowers', 'fantasy/april-flowers', 1363),
            cls('Arma', 'super-hero/arma', 1640),
            cls('AsPerUsual', 'slice-of-life/as-per-usual', 599),
            cls('AthenaComplex', 'fantasy/athena-complex', 867),
            cls('AuraFromAnotherPlanet', 'comedy/aura-from-another-planet', 369),
            cls('AverageAdventuresOfAnAverageGirl', 'slice-of-life/average-adventures-of-an-average-girl', 401),
            cls('AXED', 'comedy/axed', 1558),
            cls('Backchannel', 'super-hero/backchannel', 1456),
            cls('BadSigns', 'comedy/bad-signs', 1623),
            cls('Bastard', 'thriller/bastard', 485),
            cls('BeforeWeKnewIt', 'romance/before-we-knew-it', 1972),
            cls('BehindTheGIFs', 'comedy/behind-the-gifs', 658),
            cls('BigJo', 'romance/big-jo', 854),
            cls('BiteMe', 'thriller/bite-me', 1019),
            cls('BitterSweetCoffee', 'challenge/bitter-sweet-coffee', 797203),
            cls('Blackened', 'challenge/blackened', 363805),
            cls('BladesOfFurry', 'romance/blades-of-furry', 2383),
            cls('Blessed', 'drama/blessed', 1193),
            cls('BloodInk', 'action/blood-ink', 1490),
            cls('BloodlessWars', 'sf/bloodless-wars', 1622),
            cls('BloopBloopRelationshipComic', 'challenge/bloop-bloop-relationship-comic', 239970),
            cls('Bluechair', 'slice-of-life/bluechair', 199),
            cls('BOOItsSex', 'slice-of-life/boo-its-sex', 1413),
            cls('BoyfriendOfTheDead', 'comedy/boyfriend-of-the-dead', 1102),
            cls('BrassAndSass', 'romance/brass-and-sass', 1652),
            cls('BrimstoneAndRoses', 'romance/brimstone-and-roses', 1758),
            cls('BrothersBond', 'action/brothersbond', 1458),
            cls('BrutallyHonest', 'comedy/brutally-honest', 799),
            cls('BuzzFeedComics', 'comedy/buzzfeed-comics', 585),
            cls('CapeOfSpirits', 'action/cape-of-spirits', 1559),
            cls('CARL', 'slice-of-life/carl', 1216),
            cls('Caster', 'action/caster', 1461),
            cls('CastleSwimmer', 'fantasy/castle-swimmer', 1499),
            cls('CatchMeIfYouCan', 'challenge/catch-me-if-you-can-', 434808),
            cls('Catharsis', 'fantasy/catharsis', 396),
            cls('CatLoafAdventures', 'slice-of-life/cat-loaf-adventures', 1381),
            cls('CChansACatgirl', 'challenge/c-chans-a-catgirl', 263430),
            cls('CheeseInTheTrap', 'drama/cheese-in-the-trap', 99),
            cls('CherryBlossoms', 'romance/cherry-blossoms', 1005),
            cls('Chiller', 'thriller/chiller', 536),
            cls('CityOfBlank', 'sf/city-of-blank', 1895),
            cls('CityOfWalls', 'drama/city-of-wall', 505),
            cls('CityVamps', 'challenge/city-vamps-', 119224),
            cls('ClawShot', 'challenge/clawshot', 621465),
            cls('ClinicOfHorrors', 'supernatural/clinic-of-horrors', 3414),
            cls('ClusterFudge', 'slice-of-life/cluster-fudge', 355),
            cls('CodeAdam', 'action/code-adam', 1657),
            cls('CookingComically', 'tiptoon/cooking-comically', 622),
            cls('CrowTime', 'challenge/crow-time', 693372),
            cls('Crumbs', 'romance/crumbs', 1648),
            cls('CrystalVirusOtherStory', 'challenge/crystal-virus-other-story', 837028),
            cls('CupidsArrows', 'romance/cupids-arrows', 1538),
            cls('CursedPrincessClub', 'comedy/cursed-princess-club', 1537),
            cls('Cyberbunk', 'sf/cyberbunk', 466),
            cls('Cyberforce', 'super-hero/cyberforce', 531),
            cls('CydoniaShattering', 'fantasy/cydonia-shattering', 2881),
            cls('CykoKO', 'super-hero/cyko-ko', 560),
            cls('Darbi', 'action/darbi', 1098),
            cls('Darchon', 'challenge/darchon', 532053),
            cls('DatingWithATail', 'romance/dating-with-a-tail', 1263),
            cls('Davinchibi', 'fantasy/davinchibi', 1190),
            cls('DaYomanvilleGang', 'drama/da-yomanville-gang', 1578),
            cls('DaysOfHana', 'drama/days-of-hana', 1246),
            cls('DEADDAYS', 'horror/dead-days', 293),
            cls('Debunkers', 'challenge/debunkers', 148475),
            cls('DEEP', 'thriller/deep', 364),
            cls('Defects', 'fantasy/defects', 2731),
            cls('Denma', 'sf/denma', 921),
            cls('Dents', 'sf/dents', 671),
            cls('Deor', 'fantasy/deor', 1663),
            cls('DevilNumber4', 'supernatural/devil-no-4', 1695),
            cls('DICE', 'fantasy/dice', 64),
            cls('DistantSky', 'horror/distant-sky', 75),
            cls('DONTHATE', 'comedy/dont-hate', 1574),
            cls('DoodleForFood', 'slice-of-life/doodle-for-food', 487),
            cls('DownToEarth', 'romance/down-to-earth', 1817),
            cls('Dragnarok', 'fantasy/dragnarok', 1018),
            cls('DragnarokDescendants', 'fantasy/dragnarok-descendants', 1433),
            cls('DrawnToYou', 'challenge/drawn-to-you', 172022),
            cls('DrFrost', 'drama/dr-frost', 371),
            cls('DuelIdentity', 'challenge/duel-identity', 532064),
            cls('DungeonCleaningLife', 'action/the-dungeon-cleaning-life-of-a-once-genius-hunter', 4677),
            cls('DungeonsAndDoodlesTalesFromTheTables', 'canvas/dungeons-doodles-tales-from-the-tables', 682646),
            cls('DungeonMinis', 'challenge/dungeonminis', 64132),
            cls('Dustinteractive', 'comedy/dustinteractive', 907),
            cls('DutyAfterSchool', 'sf/duty-after-school', 370),
            cls('EatFighter', 'sports/eat-fighter', 1460),
            cls('EcstasyHearts', 'sports/ecstasy-hearts', 604),
            cls('Edith', 'romance/edith', 1536),
            cls('Eggnoid', 'sf/eggnoid', 1229),
            cls('Eleceed', 'action/eleceed', 1571),
            cls('Elena', 'horror/elena', 484),
            cls('ElfAndWarrior', 'fantasy/elf-and-warrior', 908),
            cls('EMPYREA', 'fantasy/empyrea', 1407),
            cls('EpicV', 'comedy/epic-v', 353),
            cls('EscapeRoom', 'thriller/escape-room', 1815),
            cls('EverywhereAndNowhere', 'comedy/everywhere-and-nowhere', 1598),
            cls('FalseKnees', 'canvas/false-knees', 79544),
            cls('FAMILYMAN', 'drama/family-man', 85),
            cls('FantasySketchTheGame', 'sf/fantasy-sketch', 1020),
            cls('Faust', 'supernatural/faust', 522),
            cls('FinalRaidBoss', 'fantasy/the-final-raid-boss', 3921),
            cls('FINALITY', 'mystery/finality', 1457),
            cls('Firebrand', 'supernatural/firebrand', 877),
            cls('FirstDefense', 'challenge/first-defense', 532072),
            cls('FisheyePlacebo', 'challenge/fisheye-placebo', 101841),
            cls('Flow', 'fantasy/flow', 101),
            cls('FluffyBoyfriend', 'supernatural/fluffy-boyfriend', 1164),
            cls('ForTheSakeOfSita', 'romance/for-the-sake-of-sita', 349),
            cls('FourLeaf', 'fantasy/four-leaf', 1454),
            cls('FreakingRomance', 'romance/freaking-romance', 1467),
            cls('FridayForbiddenTales', 'thriller/friday', 388),
            cls('FutureYou', 'challenge/future-you', 288439),
            cls('GameMasters', 'challenge/game-masters', 237252),
            cls('GenshinImpact', 'challenge/genshin-impact', 242646),
            cls('Gepetto', 'sf/gepetto', 81),
            cls('GhostsAmongTheWildFlowers', 'fantasy/ghosts-over-wild-flowers', 718),
            cls('GhostTeller', 'horror/ghost-teller', 1307),
            cls('GhostTheater', 'drama/ghost-theater', 1911),
            cls('GhostWife', 'romance/ghost-wife', 1471),
            cls('GirlsHaveABlog', 'slice-of-life/girls-have-a-blog', 1052),
            cls('GirlsOfTheWilds', 'action/girls-of-the-wilds', 93),
            cls('GodOfBath', 'comedy/god-of-bath', 91),
            cls('GOSU', 'action/gosu', 1099),
            cls('GourmetHound', 'drama/gourmet-hound', 1245),
            cls('GremoryLand', 'horror/gremoryland', 1893),
            cls('GuardiansOfTheVideoGame', 'sf/guardians-of-the-video-game', 368),
            cls('HAPIBUNI', 'comedy/hapi-buni', 362),
            cls('HardcoreLevelingWarrior', 'action/hardcore-leveling-warrior', 1221),
            cls('HaveYouAnyFear', 'horror/have-you-any-fear', 1197),
            cls('Haxor', 'sf/haxor', 1325),
            cls('Heartwired', 'sf/heartwired', 1539),
            cls('HeirsGame', 'drama/heirs-game', 1445),
            cls('HeliosFemina', 'fantasy/helios-femina', 638),
            cls('HelloWorld', 'slice-of-life/hello-world', 827),
            cls('Hellper', 'fantasy/hellper', 185),
            cls('Hench', 'canvas/hench/', 857225),
            cls('HeroineChic', 'super-hero/heroine-chic', 561),
            cls('HIVE', 'thriller/hive', 65),
            cls('Hooky', 'fantasy/hooky', 425),
            cls('HoovesOfDeath', 'fantasy/hooves-of-death', 1535),
            cls('HouseOfStars', 'fantasy/house-of-stars', 1620),
            cls('HowToBeAMindReaver', 'canvas/how-to-be-a-mind-reaver', 301213),
            cls('HowToBecomeADragon', 'fantasy/how-to-become-a-dragon', 1973),
            cls('HowToLove', 'slice-of-life/how-to-love', 472),
            cls('IDontWantThisKindOfHero', 'super-hero/i-dont-want-this-kind-of-hero', 98),
            cls('IF', 'action/if', 1925),
            cls('IllusionsOfAdulting', 'slice-of-life/illusions-of-adulting', 922),
            cls('IllustratedInternet', 'comedy/illustrated-internet', 750),
            cls('ILoveYoo', 'drama/i-love-yoo', 986),
            cls('ImmortalNerd', 'slice-of-life/immortal-nerd', 579),
            cls('ImTheGrimReaper', 'supernatural/im-the-grim-reaper', 1697),
            cls('Inarime', 'super-hero/inarime', 675),
            cls('InternetExplorer', 'challenge/internet-explorer', 219164),
            cls('InTheBleakMidwinter', 'sf/in-the-bleak-midwinter', 1946),
            cls('ItsMine', 'drama/its-mine', 2010),
            cls('JackieRose', 'supernatural/jackie-rose', 613),
            cls('JingleJungle', 'slice-of-life/jingle-jungle', 282),
            cls('JustACat', 'fantasy/just-a-cat', 7633),
            cls('JustAskYuli', 'slice-of-life/just-ask-yuli', 402),
            cls('JustForKicks', 'slice-of-life/just-for-kicks', 1152),
            cls('JustFriends', 'challenge/just-friends', 190722),
            cls('JustPancakes', 'comedy/just-pancakes', 1651),
            cls('Katrina', 'challenge/katrina', 532106),
            cls('KidsAreAllRight', 'drama/kids-are-all-right', 283),
            cls('Killstagram', 'thriller/killstagram', 1971),
            cls('KindOfConfidential', 'romance/kind-of-confidential', 663),
            cls('KindOfLove', 'slice-of-life/kind-of-love', 1850),
            cls('KissItGoodbye', 'challenge/kiss-it-goodbye', 443703),
            cls('KnightRun', 'sf/knight-run', 67),
            cls('KnightUnderMyHeart', 'action/knight-under-my-heart', 4215),
            cls('Kubera', 'fantasy/kubera', 83),
            cls('LalinsCurse', 'supernatural/lalins-curse', 1601),
            cls('Lars', 'slice-of-life/lars', 358),
            cls('LateBloomer', 'romance/late-bloomer', 988),
            cls('LavenderJack', 'super-hero/lavender-jack', 1410),
            cls('LESSA', 'action/lessa', 89),
            cls('LESSA2TheCrimsonKnight', 'action/lessa-2', 507),
            cls('LetsPlay', 'romance/letsplay', 1218),
            cls('LibraryGhost', 'comedy/library-ghost', 220),
            cls('LifeOutsideTheCircle', 'drama/life-outside-the-circle', 1260),
            cls('LittleMatchaGirl', 'fantasy/little-matcha-girl', 1665),
            cls('LiveForever', 'thriller/live-forever', 1312),
            cls('LiveWithYourself', 'comedy/live-with-yourself', 919),
            cls('Lone', 'fantasy/lone', 1929),
            cls('Lookism', 'drama/lookism', 1049),
            cls('LoreOlympus', 'romance/lore-olympus', 1320),
            cls('Lorna', 'slice-of-life/lorna', 1284),
            cls('LostInTranslation', 'drama/lost-in-translation', 1882),
            cls('LoveAdviceFromTheGreatDukeOfHell', 'comedy/love-advice', 1498),
            cls('LoveMeKnot', 'romance/love-me-knot', 2224),
            cls('Lozolz', 'tiptoon/lozolz', 1268),
            cls('LUFF', 'romance/luff', 1489),
            cls('Luggage', 'fantasy/luggage', 1642),
            cls('LUMINE', 'fantasy/lumine', 1022),
            cls('Lunarbaboon', 'slice-of-life/lunarbaboon', 523),
            cls('MageAndDemonQueen', 'comedy/mage-and-demon-queen', 1438),
            cls('MageAndMimic', 'comedy/mage-and-mimic', 5973),
            cls('Magical12thGraders', 'super-hero/magical-12th-graders', 90),
            cls('Magician', 'fantasy/magician', 70),
            cls('MagicSodaPop', 'fantasy/magic-soda-pop', 1947),
            cls('Magika', 'challenge/magika', 532116),
            cls('MarryMe', 'romance/marry-me', 1951),
            cls('MatchmakerHero', 'sf/matchmaker-hero', 1569),
            cls('MelvinasTherapy', 'horror/melvinas-therapy', 1021),
            cls('MeowMan', 'comedy/meow-man', 1677),
            cls('MercWorks', 'slice-of-life/mercworks', 426),
            cls('Messenger', 'fantasy/messenger', 1382),
            cls('MetaphoricalHER', 'drama/metaphorical-her', 1475),
            cls('MidnightPoppyLand', 'romance/midnight-poppy-land', 1798),
            cls('MidnightRain', 'drama/midnight-rain', 1797),
            cls('MidnightRhapsody', 'slice-of-life/midnight-rhapsody', 116),
            cls('MidnightRhapsodySeason2', 'slice-of-life/midnight-rhapsody-season2', 365),
            cls('Miez', 'sf/miez', 2719),
            cls('MissAbbottAndTheDoctor', 'romance/miss-abbott-and-the-doctor', 707),
            cls('MonsterIsle', 'challenge/monster-isle', 531999),
            cls('MOONBEARD', 'comedy/moon-beard', 471),
            cls('MoonYou', 'supernatural/moonyou', 1340),
            cls('Murrz', 'slice-of-life/murrz', 1281),
            cls('Muted', 'supernatural/muted', 1566),
            cls('MyAssassinGirlfriend', 'challenge/my-assassin-girlfriend', 249007),
            cls('MyBoo', 'supernatural/my-boo', 1185),
            cls('MyDearColdBloodedKing', 'romance/my-dear-cold-blooded-king', 961),
            cls('MyDeepestSecret', 'thriller/my-deepest-secret', 1580),
            cls('MyDictatorBoyfriend', 'comedy/my-dictator-boyfriend', 1391),
            cls('MyDragonGirlfriend', 'challenge/my-dragon-girlfriend', 162918),
            cls('MyGiantNerdBoyfriend', 'slice-of-life/my-giant-nerd-boyfriend', 958),
            cls('MyKittyAndOldDog', 'slice-of-life/my-kitty-and-old-dog', 184),
            cls('MyNameIsBenny', 'slice-of-life/my-name-is-benny', 1279),
            cls('MySClassHunter', 'action/my-s-class-hunters', 3963),
            cls('MythicItemObtained', 'fantasy/mythic-item-obtained', 4582),
            cls('MyWallflowerKiss', 'challenge/my-wallflower-kiss', 151869),
            cls('MyWifeIsAHalfDragon', 'canvas/my-wife-is-a-half-dragon', 1013669),
            cls('NanoList', 'sf/nano-list', 700),
            cls('NationalDogDay2016', 'slice-of-life/national-dog-day', 747),
            cls('NewLifeProject', 'comedy/new-life-project', 279),
            cls('Newman', 'fantasy/newman', 405),
            cls('NewNormalClass8', 'drama/new-normal-class-8', 100),
            cls('Nicholalala', 'slice-of-life/nicholalala', 418),
            cls('Noblesse', 'action/noblesse', 87),
            cls('NoblesseRaisAdventure', 'action/noblesse-spin-off', 608),
            cls('NoScope', 'sports/no-scope', 1572),
            cls('NotEvenBones', 'thriller/not-even-bones', 1756),
            cls('NothingSpecial', 'fantasy/nothing-special', 1188),
            cls('NotSoLucky', 'challenge/not-so-lucky', 673387),
            cls('OddGirlOut', 'drama/odd-girl-out', 1420),
            cls('OhHoly', 'romance/oh-holy', 809),
            cls('OmniscientReader', 'action/omniscient-reader', 2154),
            cls('ORANGEMARMALADE', 'romance/orange-marmalade', 97),
            cls('Outrage', 'super-hero/outrage', 1450),
            cls('PacificRimAmara', 'sf/pacific-rim-amara', 1327),
            cls('PandorasBlogs', 'challenge/pandoras-blogs', 532007),
            cls('PaperRoses', 'challenge/paper-roses', 39736),
            cls('PenguinLovesMev', 'slice-of-life/penguin-loves-mev', 86),
            cls('Petrichor', 'challenge/petrichor', 100835),
            cls('PhantomParadise', 'fantasy/phantom-paradise', 1250),
            cls('Phase', 'romance/phase', 2117),
            cls('Pigminted', 'slice-of-life/pigminted', 482),
            cls('PinchPoint', 'challenge/pinch-point-reborn', 334640),
            cls('Plum', 'sports/plum', 1605),
            cls('Polidiocy', 'comedy/polidiocy', 676),
            cls('Pound', 'action/pound', 1496),
            cls('PowerBallad', 'super-hero/power-ballad', 987),
            cls('Punderworld', 'challenge/punderworld', 312584),
            cls('PurpleHyacinth', 'mystery/purple-hyacinth', 1621),
            cls('RandomChat', 'drama/random-chat', 1669),
            cls('RANDOMPHILIA', 'comedy/randomphilia', 386),
            cls('Rebirth', 'sf/rebirth', 1412),
            cls('RefundHighSchool', 'fantasy/refundhighschool', 1360),
            cls('ReturnToPlayer', 'action/return-to-player', 2574),
            cls('RiseFromAshes', 'supernatural/rise-from-ashes', 959),
            cls('RoarStreetJournal', 'slice-of-life/roar-street-journal', 397),
            cls('RoomOfSwords', 'sf/room-of-swords', 1261),
            cls('RotAndRuin', 'horror/rot-and-ruin', 1878),
            cls('SafelyEndangered', 'comedy/safely-endangered', 352),
            cls('SaltyStudio', 'romance/salty-studio', 74),
            cls('SaphieTheOneEyedCat', 'slice-of-life/saphie-one-eyed-cat', 670),
            cls('SAVEME', 'drama/bts-save-me', 1514),
            cls('ScoobandShag', 'challenge/scoob-and-shag', 210827),
            cls('ScorchingRomance', 'romance/scorching-romance', 1662),
            cls('Seed', 'sf/seed', 1480),
            cls('SHADOW', 'super-hero/shadow', 281),
            cls('ShadowChildren', 'challenge/shadow-children', 532144),
            cls('ShadowPirates', 'action/shadow-pirates', 1455),
            cls('Shard', 'supernatural/shard', 960),
            cls('Shiloh', 'thriller/shiloh', 1649),
            cls('ShootAround', 'drama/shoot-around', 399),
            cls('Shriek', 'thriller/shriek', 772),
            cls('SID', 'supernatural/sid', 497),
            cls('SIDEKICKS', 'super-hero/sidekicks', 92),
            cls('SimonSues', 'supernatural/simon-sues', 1619),
            cls('SirensLament', 'romance/sirens-lament', 632),
            cls('Sithrah', 'fantasy/sithrah', 524),
            cls('SkateFire100', 'sports/skate-fire-100', 1674),
            cls('SmallWorld', 'slice-of-life/small-world', 1159),
            cls('SmileBrush', 'slice-of-life/smile-brush', 94),
            cls('SmileBrushMyOldPictures', 'slice-of-life/smile-brush-my-old-pictures', 302),
            cls('Snailogy', 'slice-of-life/snailogy', 387),
            cls('SOLEIL', 'fantasy/soleil', 1823),
            cls('SOULCARTEL', 'fantasy/soul-cartel', 72),
            cls('SoulOnHold', 'supernatural/soul-on-hold', 1701),
            cls('SpaceBoy', 'sf/space-boy', 400),
            cls('SpaceVixen', 'challenge/space-vixen-deep-space-k9', 207049),
            cls('SpellsFromHell', 'fantasy/spells-from-hell', 2431),
            cls('SpiritFingers', 'drama/spirit-fingers', 1577),
            cls('Spirits', 'fantasy/spirits-re', 1348),
            cls('StalkerXStalker', 'challenge/stalker-x-stalker', 245662),
            cls('STARCROSS', 'super-hero/star-cross', 1599),
            cls('StayingHealthyTogether', 'tiptoon/staying-healthy-together', 1963),
            cls('StrawberrySeafoam', 'fantasy/strawberry-seafoam', 1248),
            cls('SubtleDisaster', 'drama/subtle-disaster', 350),
            cls('SubZero', 'romance/subzero', 1468),
            cls('SuitorArmor', 'fantasy/suitor-armor', 2159),
            cls('SuperSecret', 'romance/super-secret', 666),
            cls('SupersonicGirl', 'super-hero/supersonic-girl', 633),
            cls('SweetHome', 'thriller/sweethome', 1285),
            cls('SwimmingLessonsForAMermaid', 'romance/swimming-lessons-for-a-mermaid', 1912),
            cls('SwordInterval', 'supernatural/sword-interval', 486),
            cls('TalesOfTheUnusual', 'horror/tales-of-the-unusual', 68),
            cls('TheAdvancedPlayerOfTheTutorialTower', 'action/the-advanced-player-of-the-tutorial-tower', 2409),
            cls('TheBadguys', 'super-hero/the-bad-guys', 701),
            cls('TheBrooklynite', 'super-hero/the-brooklynite', 813),
            cls('TheCliff', 'thriller/the-cliff', 80),
            cls('TheCroaking', 'fantasy/the-croaking', 1494),
            cls('TheDaneMen', 'comedy/the-danemen', 395),
            cls('TheDevilIsAHandsomeMan', 'drama/the-devil-is-a-handsome-man', 1311),
            cls('TheDoctorsAreOut', 'romance/the-doctors-are-out', 1910),
            cls('TheFeverKing', 'super-hero/the-fever-king', 1659),
            cls('TheFourOfThem', 'drama/the-four-of-them', 1524),
            cls('TheGamer', 'action/the-gamer', 88),
            cls('TheGentlemansArmchair', 'comedy/the-gentlemans-armchair', 469),
            cls('TheGirlDownstairs', 'romance/the-girl-downstairs', 1809),
            cls('THEGIRLFROMCLASS', 'drama/the-girl-from-class', 73),
            cls('TheGodOfHighSchool', 'action/the-god-of-high-school', 66),
            cls('TheGreenhouse', 'challenge/the-greenhouse-gl', 278312),
            cls('TheKissBet', 'romance/the-kiss-bet', 1617),
            cls('TheLifeOfTheThreeBears', 'slice-of-life/the-life-of-the-three-bears', 390),
            cls('TheLittleTrashmaid', 'canvas/the-little-trashmaid', 300138),
            cls('ThePurpleHeart', 'super-hero/the-purple-heart', 723),
            cls('TheRedBook', 'horror/the-red-book', 467),
            cls('TheRedHook', 'super-hero/the-red-hook', 643),
            cls('TheRedKing', 'supernatural/the-red-king', 1687),
            cls('TheRoomies', 'challenge/the-roomies-archive', 513669),
            cls('TheShadowProphet', 'drama/the-shadow-prophet', 1881),
            cls('TheSoundOfYourHeart', 'comedy/the-sound-of-your-heart', 269),
            cls('TheSpectrumOfUs', 'challenge/the-spectrum-of-us', 334525),
            cls('TheSteamDragonExpress', 'fantasy/steam-dragon-express', 1270),
            cls('TheStoriesOfThoseAroundMe', 'romance/the-stories-of-those-around-me', 96),
            cls('TheStrangeTalesOfOscarZahn', 'fantasy/the-strange-tales-of-oscar-zahn', 685),
            cls('TheVaultOfHorrorACollectionOfNightmares', 'horror/the-vault-of-horror-a-collection-of-nightmares', 295),
            cls('TheWeeklyRoll', 'challenge/the-weekly-roll', 358889),
            cls('TheWeightOfOurSky', 'historical/the-weight-of-our-sky', 1739),
            cls('TheWitchAndTheBull', 'fantasy/the-witch-and-the-bull', 1892),
            cls('TheWolfAndRedRidingHood', 'comedy/wolf-and-red-riding-hood', 2142),
            cls('TheWolfmanOfWulvershire', 'mystery/the-wolfman-of-wulvershire', 1784),
            cls('TheWorldWhereIBelong', 'supernatural/the-world-where-i-belong', 1318),
            cls('TheWrathAndTheDawn', 'fantasy/the-wrath-and-the-dawn', 1772),
            cls('ThirdShiftSociety', 'supernatural/third-shift-society', 1703),
            cls('Thornstone', 'fantasy/thornstone', 1612),
            cls('TickleTown', 'comedy/tickle-town', 428),
            cls('ToasterDude', 'comedy/toaster-dude', 1983),
            cls('TokyoThreatDocumentationProject', 'challenge/tokyo-threat-documentation-project', 417973),
            cls('TowerOfGod', 'fantasy/tower-of-god', 95),
            cls('TrailerParkWarlock', 'comedy/trailer-park-warlock', 1512),
            cls('TrashBird', 'comedy/trash-bird', 473),
            cls('TrueBeauty', 'romance/truebeauty', 1436),
            cls('Trump', 'fantasy/trump', 84),
            cls('UndeadEd', 'supernatural/undeaded', 468),
            cls('UnderPrin', 'supernatural/underprin', 78),
            cls('UnderTheAegis', 'fantasy/under-the-aegis', 436),
            cls('UnholyBlood', 'supernatural/unholy-blood', 1262),
            cls('UnintentionalGame', 'challenge/unintentional-game', 162674),
            cls('UnknownCaller', 'thriller/ar-toon', 775),
            cls('UnlovableReplacement', 'romance/unlovable-replacement', 1762),
            cls('UnluckyIsAsLuckyDoes', 'comedy/unlucky-is-as-lucky-does', 1554),
            cls('UnOrdinary', 'super-hero/unordinary', 679),
            cls('UnTouchable', 'romance/untouchable', 79),
            cls('UpAndOut', 'slice-of-life/up-and-out', 488),
            cls('UrbanAnimal', 'super-hero/urban-animal', 1483),
            cls('Uriah', 'horror/uriah', 1607),
            cls('VampireFamily', 'comedy/vampire-family', 6402),
            cls('VarsityNoir', 'mystery/varsity-noir', 1613),
            cls('VersionDayAndNight', 'drama/version-day-and-night', 1796),
            cls('WafflesAndPancakes', 'slice-of-life/waffles-and-pancakes', 1310),
            cls('WarCry', 'super-hero/war-cry', 1247),
            cls('WarningLabel', 'romance/warning-label', 1051),
            cls('Watermelon', 'fantasy/watermelon', 1435),
            cls('WaywardLegends', 'challenge/wayward-legends', 533029),
            cls('WaywardSons', 'challenge/wayward-sons', 533024),
            cls('WeakHero', 'action/weakhero', 1726),
            cls('WEBTOONGREENLiGHT', 'action/webtoon-greenlight', 1988),
            cls('WestwoodVibrato', 'drama/westwood-vibrato', 537),
            cls('WhereTangentsMeet', 'romance/where-tangents-meet', 421),
            cls('WhiteWidow', 'challenge/white-widow', 529691),
            cls('WindBreaker', 'sports/wind-breaker', 372),
            cls('WinterMoon', 'fantasy/winter-moon', 1093),
            cls('WinterWoods', 'drama/winter-woods', 344),
            cls('WitchCreekRoad', 'horror/witch-creek-road', 1453),
            cls('WitchHunt', 'supernatural/witch-hunt', 363),
            cls('Wolfsbane', 'horror/wolfsbane', 1826),
            cls('WorldsStrongestTroll', 'fantasy/worlds-strongest-troll', 5315),
            cls('XINK3R', 'super-hero/xinker', 541),
            cls('YourAdventure', 'comedy/your-adventure', 506),
            cls('YourLetter', 'drama/your-letter', 1540),
            cls('YouveGottaBeKittenMe', 'challenge/youve-gotta-be-kitten-me', 383661),
            cls('YumisCells', 'slice-of-life/yumi-cell', 478),
            cls('YunaAndKawachan', 'drama/yuna-and-kawachan', 1840),
            cls('ZeroGame', 'fantasy/zero-game', 1704),
            cls('ZomCom', 'challenge/zomcom', 70195),
        )



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
import collections
import functools
import logging
import random
import time
from typing import Any
from urllib import parse, robotparser

import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from . import configuration

# Default number of retries
MaxRetries = 3

# Factor for retry backoff (see urllib3.util.retry, this default means
# 2s, 4s, 8s)
RetryBackoffFactor = 2

# Default connection timeout
ConnectionTimeoutSecs = 60

logger = logging.getLogger(__name__)


class Session(requests.Session):
    """This session implements a very simple host-based throttling system: For
    each hostname we see, we keep a record on when is the earliest time we want
    to send the next request: If before sending a request this time isn't
    reached, we sleep a bit until the requirements are satisfied. By default,
    we only delay a random amount of at most 0.3sec - but some hosts might need
    longer delays.
    """
    def __init__(self) -> None:
        super().__init__()

        retry = Retry(MaxRetries, backoff_factor=RetryBackoffFactor)
        self.mount('http://', HTTPAdapter(max_retries=retry))
        self.mount('https://', HTTPAdapter(max_retries=retry))
        self.headers.update({
            'Accept-Language': 'en',
            'User-Agent': configuration.UserAgent,
        })

        self.throttles: dict[str, RandomThrottle] = collections.defaultdict(
            lambda: RandomThrottle())
        self.host_options: dict[str, dict[Any, Any]] = {}

    def send(self, request, **kwargs):
        if 'timeout' not in kwargs:
            kwargs['timeout'] = ConnectionTimeoutSecs

        hostname = parse.urlsplit(request.url).hostname
        self.throttles[hostname].delay()
        if hostname in self.host_options:
            kwargs.update(self.host_options[hostname])

        return super().send(request, **kwargs)

    def add_throttle(self, hostname: str, th_min, th_max):
        """Adds a new throttle for a host: Might overwrite the existing one.
        """
        self.throttles[hostname] = RandomThrottle(th_min, th_max)

    def add_host_options(self, hostname: str, options):
        """Adds custom options for a specific host: Might overwrite the existing one.
        """
        self.host_options[hostname] = options


class RandomThrottle:
    def __init__(self, th_min=0.0, th_max=0.3) -> None:
        self.th_min = th_min
        self.th_max = th_max
        self.next = time.time()

    def delay(self):
        d = self.next - time.time()
        if d > 0:
            time.sleep(d)
        self.next = time.time() + random.uniform(self.th_min, self.th_max)


def check_robotstxt(url: str, session: Session) -> None:
    """Check if robots.txt allows our user agent for the given URL.
    @raises: IOError if URL is not allowed
    """
    roboturl = _get_roboturl(url)
    rp = _get_robotstxt_parser(roboturl, session)
    if not rp.can_fetch(configuration.App, str(url)):
        raise IOError("%s is disallowed by %s" % (url, roboturl))


def _get_roboturl(url: str) -> str:
    """Get robots.txt URL from given URL."""
    pu = parse.urlsplit(url)
    return parse.urlunsplit((pu.scheme, pu.netloc, "/robots.txt", None, None))


@functools.lru_cache()
def _get_robotstxt_parser(url, session: Session) -> robotparser.RobotFileParser:
    """Get a RobotFileParser for the given robots.txt URL."""
    rp = robotparser.RobotFileParser()
    try:
        req = session.get(url)
    except Exception as e:
        # connect or timeout errors are treated as an absent robots.txt
        rp.allow_all = True
        logger.trace('GET %r caused exception: %s', url, e)
    else:
        if req.status_code >= 400:
            rp.allow_all = True
            logger.trace('GET %r caused HTTP error %i', url, req.status_code)
        elif req.status_code == 200:
            rp.parse(req.text.splitlines())
            logger.trace('GET %r successful, %i bytes', url, len(req.content))
            if rp.default_entry:
                # Filter default deny rules
                de = rp.default_entry
                de.rulelines = [rule for rule in de.rulelines if rule.allowance]
    return rp


# A default session for cookie and connection sharing
default_session = Session()



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
import os
from re import IGNORECASE, compile

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import (
    ComicControlScraper,
    WordPressNavi,
    WordPressScraper,
    WordPressWebcomic,
)


class EarthsongSaga(_ParserScraper):
    stripUrl = 'http://earthsongsaga.com/vol%s'
    url = stripUrl % '5/epilogue5.php'
    firstStripUrl = stripUrl % '1/vol1cover.php'
    imageSearch = '//img[contains(@src, "images/vol")]'
    prevSearch = ('//a[@title="Previous"]', '//td[@width=98]//a')
    endOfLife = True

    def namer(self, image_url, page_url):
        imgmatch = compile(r'images/vol(\d+)/ch(?:apter)?(\d+)/(.*)\.\w+$',
                           IGNORECASE).search(image_url)
        if imgmatch:
            return 'vol%02d_ch%02d_%s' % (int(imgmatch.group(1)),
                int(imgmatch.group(2)), imgmatch.group(3))
        imgmatch = compile(r'images/vol(\d+)/[^/]*cover[^/]*$',
                           IGNORECASE).search(image_url)
        return 'vol%02dcover' % (int(imgmatch.group(1)))

    def getPrevUrl(self, url, data):
        # Fix wrong navigation links
        if url == self.stripUrl % '1/63.php':
            return self.stripUrl % '1/62.php'
        elif url == self.stripUrl % '2/vol2cover.html':
            return self.stripUrl % '1/121.php'
        elif url == self.stripUrl % '3/1.html':
            return self.stripUrl % '3/ch7cover.html'

        return super().getPrevUrl(url, data)


class EatLiver(_ParserScraper):
    url = 'http://www.eatliver.com/'
    starter = indirectStarter
    multipleImagesPerStrip = True
    imageSearch = '//div[d:class("post-content")]//img'
    prevSearch = '//a[@rel="prev"]'
    latestSearch = '//a[@rel="bookmark"]'


class EatThatToast(WordPressScraper):
    url = 'http://eatthattoast.com/'
    firstStripUrl = url + 'comic/thewizard/'
    textSearch = WordPressScraper.imageSearch + '/@alt'


class EdibleDirt(_BasicScraper):
    url = 'http://eddirt.frozenreality.co.uk/'
    stripUrl = url + 'index.php?id=%s'
    firstStripUrl = stripUrl % '0'
    imageSearch = compile(tagre("img", "src", r'(strips/[^"]+)'))
    prevSearch = compile(tagre("a", "href", r"(index\.php\?id=\d+)") +
                         "Previous")
    help = 'Index format: number'


class EdmundFinney(WordPressNavi):
    url = 'http://eqcomics.com/'
    firstStripUrl = url + '2009/03/08/sunday-aliens/'


class ElfOnlyInn(_BasicScraper):
    url = 'http://www.elfonlyinn.net/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '20020523'
    imageSearch = compile(tagre("img", "src", r'(/comics/[^"]+)'))
    prevSearch = compile(tagre("a", "href", r'(/d/\d+\.html)') +
                         tagre("img", "src", r'/images/previous_day\.gif'))
    help = 'Index format: yyyymmdd'


class ElGoonishShive(ComicControlScraper):
    url = 'http://www.egscomics.com/'


class ElGoonishShiveNP(ComicControlScraper):
    url = 'http://www.egscomics.com/egsnp.php'


class EmergencyExit(_BasicScraper):
    url = 'http://www.eecomics.net/'
    stripUrl = url + "?strip_id=%s"
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(r'"(comics/.+?)"')
    prevSearch = compile(tagre("a", "href", r'(\?strip_id=\d+)') + tagre("img", "alt", r"Prior"))
    help = 'Index format: n'


class Erfworld(ParserScraper):
    stripUrl = 'https://archives.erfworld.com/%s'
    url = stripUrl % 'getLatestPage.php'
    firstStripUrl = stripUrl % 'Kickstarter+Stories/1'
    imageSearch = '//div[@class="page_content"]//img'
    textSearch = '//div[@class="page_content"]'
    prevSearch = '//li[@class="previous"]/a'
    nextSearch = '//li[@class="next"]/a'
    multipleImagesPerStrip = True
    textOptional = True
    starter = bounceStarter

    def shouldSkipUrl(self, url, data):
        """Skip pages without images."""
        return not self.match(data, self.imageSearch)

    def namer(self, image_url, page_url):
        # Fix inconsistent filenames
        filename = util.urlpathsplit(image_url)[-1]
        page = util.urlpathsplit(page_url.replace('+', '-'))
        return '_'.join(page[:1] + [filename])

    def getPrevUrl(self, url, data):
        # Fix missing navigation links between books
        if url == self.stripUrl % 'Book+5/1':
            return self.stripUrl % 'Book+4/203'
        elif url == self.stripUrl % 'Book+4/1':
            return self.stripUrl % 'Book+3/145'
        elif url == self.stripUrl % 'Book+3/1':
            return self.stripUrl % 'Book+2/231'
        elif url == self.stripUrl % 'Book+2/1':
            return self.stripUrl % 'Book+1/184'
        elif url == self.stripUrl % 'Book+1/1':
            return self.stripUrl % 'Book+0/81'
        elif url == self.stripUrl % 'Book+0/1':
            return self.stripUrl % 'Kickstarter+Stories/54'
        return super().getPrevUrl(url, data)


class ErmaFelnaEDF(ParserScraper):
    stripUrl = 'https://www.stevegallacci.com/archive/edf/%s'
    firstStripUrl = stripUrl % '0000/00/00'
    url = firstStripUrl
    imageSearch = '//img[@id="comic-container"]'
    prevSearch = '//a[@title="Previous Comic"]'
    latestSearch = '//a[@title="Current Comic"]'
    starter = indirectStarter

    def namer(self, image_url, page_url):
        # Fix inconsistent filenames
        postDate = '-'.join(util.urlpathsplit(page_url)[-3:])
        filename = util.urlpathsplit(image_url)[-1]
        return f'{postDate}_{filename}'


class ErrantStory(_BasicScraper):
    url = 'http://www.errantstory.com/'
    stripUrl = url + '%s'
    imageSearch = compile(r'<img[^>]+?src="([^"]*?comics/.+?)"')
    prevSearch = compile(r'><a href="(.+?)">&lt;Previous</a>')
    help = 'Index format: yyyy-mm-dd/num'


class Erstwhile(WordPressNavi):
    url = 'http://www.erstwhiletales.com/'
    endOfLife = True


class EverybodyLovesEricRaymond(_ParserScraper):
    url = 'http://geekz.co.uk/lovesraymond/'
    firstStripUrl = url + 'archive/slashdotted'
    imageSearch = '//div[d:class("entry-content")]//img'
    prevSearch = '//a[@rel="prev"]'


class EvilDiva(WordPressScraper):
    url = ('https://web.archive.org/web/20190221223751/'
        'https://www.evildivacomics.com/')
    firstStripUrl = url + 'comic/evil-diva-issue-1-cover/'
    endOfLife = True


class EvilInc(ParserScraper):
    url = 'https://www.evil-inc.com/'
    imageSearch = ('//div[@id="unspliced-comic"]/img',
        '//div[@id="unspliced-comic"]/picture//img')
    prevSearch = '//a[./i[d:class("fa-chevron-left")]]'
    firstStripUrl = url + 'comic/monday-3/'


class Evilish(ParserScraper):
    url = ('https://web.archive.org/web/20220125053115/'
        'http://evilish.pensandtales.com/')
    stripUrl = url + '?date=%s'
    firstStripUrl = stripUrl % '20020630'
    imageSearch = '//img[@alt="Today\'s Comics"]'
    prevSearch = '//a[img[@alt="Previous Comic"]]'
    endOfLife = True
    help = 'Index format: yyyymmdd'


class Evon(WordPressScraper):
    url = 'http://evoncomics.com/'
    stripUrl = url + '?comic=%s'
    firstStripUrl = stripUrl % 'chapter-1'
    adult = True


class ExorcismAcademy(WordPressWebcomic):
    url = 'https://ea.asmodrawscomics.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'title-page'
    multipleImagesPerStrip = True
    adult = True
    namer = joinPathPartsNamer(imageparts=range(-3, 0))


class ExploitationNow(WordPressNavi):
    url = 'http://www.exploitationnow.com/'
    firstStripUrl = url + '2000-07-07/9'
    endOfLife = True


class Exvulnerum(_ParserScraper):
    url = 'https://zules.com/exvulnerum/'
    stripUrl = url + 'comic.php?page=%s'
    firstStripUrl = stripUrl.replace('comic', 'newprologue') % '1'
    imageSearch = ('//img[contains(@src, "pages/")]',
                   '//img[contains(@src, "newprologue/")]')
    prevSearch = '//a[./img[contains(@src, "nav_prev")]]'


class ExtraFabulousComics(WordPressScraper):
    url = 'https://extrafabulouscomics.com/'
    firstStripUrl = url + 'comic/buttfly/'
    starter = bounceStarter
    multipleImagesPerStrip = True

    def namer(self, image_url, page_url):
        imagename = os.path.basename(image_url)
        pagepart = compile(r'/comic/([^/]+)/$').search(page_url).group(1)
        return '_'.join((pagepart, imagename))

    def shouldSkipUrl(self, url, data):
        return self.match(data, '//div[@id="comic"]//iframe')


class ExtraLife(_BasicScraper):
    url = 'http://www.myextralife.com/'
    stripUrl = url + 'comic/%s/'
    imageSearch = compile(tagre("img", "src", r'(http://www\.myextralife\.com/wp-content/uploads/[^"]+)', before="comic"))
    prevSearch = compile(tagre("a", "href", r'([^"]+)', before="prev_comic"))
    help = 'Index format: stripname'


class ExtraOrdinary(_ParserScraper):
    url = 'https://www.exocomics.com/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '01'
    prevSearch = '//a[d:class("prev")]'
    imageSearch = '//img[d:class("image-style-main-comic")]'
    help = 'Index format: number'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2019 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from .. import util
from .common import WordPressSpliced


class _CommonMulti(WordPressSpliced):
    def __init__(self, name, path, first, eol=False):
        super().__init__(name)
        self.url = self.baseUrl + 'series/' + path + '/'
        self.firstStripUrl = self.stripUrl % first
        self.endOfLife = eol


class AlienDice(WordPressSpliced):
    url = 'https://aliendice.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % '05162001'

    def shouldSkipUrl(self, url, data):
        """Skip pages without images."""
        return not self.match(data, self.imageSearch)

    def getPrevUrl(self, url, data):
        # Fix broken navigation
        if url == self.stripUrl % 'day-29-part-2-page-3-4':
            return self.stripUrl % 'day-29-part-2-page-3-2'
        return super().getPrevUrl(url, data)

    def namer(self, image_url, page_url):
        # Fix inconsistent filename
        return util.urlpathsplit(image_url)[-1].replace('20010831', '2001-08-31')


class AlienDiceLegacy(WordPressSpliced):
    name = 'AlienDice/Legacy'
    baseUrl = 'https://aliendice.com/'
    url = baseUrl + 'series/legacy/'
    stripUrl = baseUrl + 'comic/%s/'
    firstStripUrl = stripUrl % 'legacy-1'
    endOfLife = True

    def isfirststrip(self, url):
        # Strip series identifier
        return super().isfirststrip(url.rsplit('?', 1)[0])


class TheCyantianChronicles(_CommonMulti):
    baseUrl = 'https://cyantian.net/'

    def __init__(self, name, path, first, sid, eol=False):
        self.stripUrl = self.baseUrl + f'%s/?sid={sid}'
        super().__init__('TheCyantianChronicles/' + name, path, first, eol)

    @classmethod
    def getmodules(cls):
        return (
            cls('Akaelae', 'akaelae', '05182003', 15043, eol=True),
            cls('Artwork', 'art-gallery', '07162003', 15102),
            cls('CampusSafari', 'original-campus-safari', '10012000', 13804, eol=True),
            cls('CampusSafariReboot', 'campus-safari', 'campus-safari-chapter-0', 13790),
            cls('CesileesDiary', 'cesilees-diary', '12062001-2', 16726, eol=True),
            cls('Darius', 'darius', '03102010', 14353, eol=True),
            cls('DracoVulpes', 'draco-vulpes', 'draco-vulpes', 13788),
            cls('GenoworksSaga', 'genoworks-saga', '07012004', 13794),
            cls('GralenCraggHall', 'kiet', '07152002', 13798, eol=True),
            cls('Kiet', 'kiet-2', 'kiet-c01', 14351),
            cls('NoAngel', 'no-angel', '08112001', 16644, eol=True),
            cls('RandomRamblings', 'gallery', 'cookie-war', 13801),
            cls('SinkOrSwim', 'sink-or-swim', '05112001', 13796, eol=True),
            cls('VincentAndFilaire', 'vincent-and-filaire', 'vincent-and-filaire', 13792),
        )


class Shivae(WordPressSpliced):
    url = 'https://shivae.net/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % '2002-02-27'


class ShivaeComics(_CommonMulti):
    baseUrl = 'https://shivae.net/'

    def __init__(self, name, path, first, sid, eol=False):
        self.stripUrl = self.baseUrl + f'comic/%s/?sid={sid}'
        super().__init__('Shivae/' + name, path, first, eol)

    @classmethod
    def getmodules(cls):
        return (
            cls('Pure', 'pure-2', '2002-02-27', 1328, eol=True),
            cls('SerinFairyHunter', 'pure', 'character-serin', 1289),
            cls('SivineBlades', 'sivine-blades', '2002-06-30', 1326, eol=True),
        )



#!/usr/bin/env python3
# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
"""
Script to get a list of gocomics and save the info in a JSON file for further
processing.
"""

from scriptutil import ComicListUpdater


class GoComicsUpdater(ComicListUpdater):
    dup_templates = (
        "ComicsKingdom/%s",
    )

    # names of comics to exclude
    excluded_comics = (
        # Has its own module
        'Widdershins',
        # Moved to webtoons
        "FalseKnees",
    )
    spanish: set[str] = set()

    def handle_atozlist(self, url: str) -> None:
        """Parse one GoComics alphabetic page."""
        data = self.get_url(url, expand=False)

        for comiclink in self.xpath(data, '//a[d:class_start("ComicsAtoZ_comics__link_")]'):
            link = comiclink.attrib['href'].split('/')[1].strip()
            name = comiclink.xpath('.//h3')[0].text
            self.add_comic(name, (link, self.detect_lang(name, link)))

    def detect_lang(self, name: str, link: str) -> str | None:
        '''Language heuristics'''
        if ("en Español" in name or
                "spanish" in link or "espanol" in link or
                link in self.spanish):
            return "es"
        return None

    def find_spanish(self) -> None:
        data = self.get_url('https://www.gocomics.com/comics', expand=False)
        for comiclink in self.xpath(data, '//section[.//h2[contains(text(), "en Español")]]//a'):
            self.spanish.add(comiclink.attrib['href'].split('/')[1].strip())

    def collect_results(self) -> None:
        """Parse all listing pages."""
        self.find_spanish()
        self.handle_atozlist('https://www.gocomics.com/comics/a-to-z')
        self.handle_atozlist('https://www.gocomics.com/political-cartoons/political-a-to-z')

    def get_entry(self, name: str, data: tuple[str, str]) -> str:
        url, lang = data
        langopt = ", '%s'" % lang if lang else ''
        return u"cls('%s', '%s'%s)," % (name, url, langopt)


if __name__ == '__main__':
    GoComicsUpdater(__file__).run()



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from re import IGNORECASE, compile, escape, sub

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import (
    ComicControlScraper,
    WordPressNavi,
    WordPressScraper,
    WordPressSpliced,
    WordPressWebcomic,
)


class SabrinaOnline(_BasicScraper):
    url = 'http://sabrina-online.com/'
    stripUrl = url + '%s.html'
    firstStripUrl = stripUrl % '1996-01'
    imageSearch = (compile(tagre("a", "href", r'(strips/[^"]*)')),
                   compile(tagre("img", "src", r'(pages/[^"]*)')))
    prevSearch = compile(tagre("a", "href", r"(\d\d\d\d-\d\d.html)") +
                         tagre("img", "src", "b_back.gif"))
    help = 'Index format: yyyy-qq'
    adult = True
    multipleImagesPerStrip = True

    def starter(self):
        """Pick last one in a list of archive pages."""
        archive = self.url + 'archive.html'
        data = self.getPage(archive)
        search = compile(tagre("a", "href", r"(\d\d\d\d-\d\d.html)"))
        archivepages = self.fetchUrls(archive, data, search)
        return archivepages[-1]


class SaffronAndSage(WordPressScraper):
    url = 'https://www.saffroncomic.com/'
    firstStripUrl = url + 'comic/p0001/'


class SailorsunOrg(WordPressScraper):
    url = 'https://sailorsun.org/'

    def shouldSkipUrl(self, url, data):
        return 'sailorsun-org-credits' in url  # video


class SamAndFuzzy(_ParserScraper):
    url = 'http://www.samandfuzzy.com/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[@class="comic-image"]'
    prevSearch = '//li[@class="prev-page"]/a'
    help = 'Index format: n (unpadded)'


class SandraOnTheRocks(ComicControlScraper):
    url = 'https://pixietrixcomix.com/sandra-on-the-rocks/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % 'start-by-running'
    endOfLife = True
    help = 'Index format: name'


class Savestate(WordPressNavi):
    url = 'http://www.savestatecomic.com/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '2014/02/pokemon-bank'


class ScandinaviaAndTheWorld(_ParserScraper):
    url = 'https://satwcomic.com/sweden-denmark-and-norway'
    firstStripUrl = url
    starter = indirectStarter
    imageSearch = '//img[@itemprop="image"]'
    prevSearch = '//a[@accesskey="p"]'
    latestSearch = '//a[contains(@title, "Latest")]'
    textSearch = '//span[@itemprop="articleBody"]'


class ScaryGoRound(_ParserScraper):
    url = ('https://web.archive.org/web/20190327203330/'
        'https://www.scarygoround.com/sgr/ar.php')
    stripUrl = url + '?date=%s'
    firstStripUrl = stripUrl % '20020604'
    imageSearch = '//img[contains(@src, "/strips/")]'
    prevSearch = '//a[contains(text(), "Previous")]'
    endOfLife = True
    help = 'Index format: yyyymmdd'


class ScenesFromAMultiverse(_BasicScraper):
    url = 'http://amultiverse.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2010/06/14/parenthood'
    imageSearch = (
        compile(tagre("div", "id", "comic") + r"\s*" +
            tagre("img", "src",
                r'(.*amultiverse.com/wp-content/uploads/\d+/\d+/[^"]+)')),
        compile(tagre("div", "id", "comic") + r"\s*" +
            tagre("a", "href", r'[^"]*') +
            tagre("img", "src",
                r'(.*amultiverse.com/wp-content/uploads/\d+/\d+/[^"]+)')),
    )
    prevSearch = compile(tagre("a", "href", r'(%scomic/\d+\d+/\d+/\d+/[^"]+)' % rurl, after="prev"))
    help = 'Index format: yyyy/mm/dd/stripname'


class SchlockMercenary(ParserScraper):
    url = 'https://www.schlockmercenary.com/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '2000-06-12'
    imageSearch = '//div[@class="strip-image-wrapper"]/img'
    prevSearch = '//a[@class="previous-strip"]'
    multipleImagesPerStrip = True
    endOfLife = True
    help = 'Index format: yyyy-mm-dd'


class SchoolBites(_ParserScraper):
    url = ('https://web.archive.org/web/20170215065523/'
        'http://schoolbites.net/')
    stripUrl = url + 'd/%s.html'
    imageSearch = '//img[d:class("ksc")]'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True
    help = 'Index format: yyyymmdd'


class Schuelert(_ParserScraper):
    url = ('https://web.archive.org/web/20190103022830/'
        'http://www.schuelert.de/')
    stripUrl = url + 'index.php?paged=%s'
    firstStripUrl = stripUrl % '3'
    imageSearch = '//img[contains(@src, "wp-content")]'
    prevSearch = '//span[d:class("prevlink")]/a'
    multipleImagesPerStrip = True
    endOfLife = True
    lang = 'de'


class Science(_ParserScraper):
    stripUrl = ('https://web.archive.org/web/20180616152753/'
        'http://sci-ence.org/%s/')
    url = stripUrl % 'new-york-comic-con-2013'
    firstStripUrl = stripUrl % 'periodic-table-element-ass'
    prevSearch = '//a[d:class("navi-prev")]'
    imageSearch = '//div[@class="comicpane"]//img'
    endOfLife = True


class SequentialArt(_ParserScraper):
    url = 'https://www.collectedcurios.com/sequentialart.php'
    stripUrl = url + '?s=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[d:class("w3-image")]'
    prevSearch = '//a[@id="backOne"]'
    help = 'Index format: name'


class SexyLosers(_ParserScraper):
    adult = True
    url = 'https://www.sexylosers.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % '003'
    imageSearch = '//div[@class="entry-content"]//img'
    prevSearch = '//a[@rel="prev"]'
    latestSearch = '//a[@rel="bookmark"]'
    help = 'Index format: nnn'
    starter = indirectStarter
    namer = joinPathPartsNamer(pageparts=(-1,), imageparts=(-1,), joinchar='-')


class ShadesOfGray(ParserScraper):
    url = 'https://www.theduckwebcomics.com/Shades_of_Gray/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '4820502'
    imageSearch = '//div[@id="comic"]/img'
    prevSearch = '//a[img[@class="arrow_prev"]]'
    nextSearch = '//a[img[@class="arrow_next"]]'
    starter = bounceStarter
    namer = joinPathPartsNamer(pageparts=(-1,))
    endOfLife = True


class Sharksplode(WordPressScraper):
    url = 'http://sharksplode.com/'
    textSearch = '//div[@id="comic"]//img/@alt'
    allow_errors = (403,)


class Sheldon(ParserScraper):
    url = 'https://www.sheldoncomics.com/'
    firstStripUrl = url + 'comic/well-who-is-this/'
    imageSearch = '//div[@id="comic"]//img/@data-src-img'
    prevSearch = '//a[img[d:class("left")]]'


class Shifters(ParserScraper):
    baseUrl = 'https://shiftersonline.com/'
    url = baseUrl + 'series/shifters-redux/'
    stripUrl = baseUrl + 'comic/%s/'
    firstStripUrl = stripUrl % 'chapter-1-pg-1'
    imageSearch = '//div[@id="spliced-comic"]//span[@class="default-lang"]//img'
    prevSearch = '//a[@class="previous-comic"]'
    latestSearch = '//div[@id="comic-archive-list"]//a'
    starter = indirectStarter
    namer = joinPathPartsNamer(pageparts=(-1,))


class ShiftersOnGossamerWings(Shifters):
    name = 'Shifters/OnGossamerWings'
    baseUrl = 'https://shiftersonline.com/'
    url = baseUrl + 'series/shifters-on-gossamer-wings/'
    stripUrl = baseUrl + 'comic/%s/'
    firstStripUrl = stripUrl % 'on-gossamer-wings-cover'


class ShiftersTheBeastWithin(Shifters):
    name = 'Shifters/TheBeastWithin'
    baseUrl = 'https://shiftersonline.com/'
    url = baseUrl + 'series/shifters-the-beast-within/'
    stripUrl = baseUrl + 'comic/%s/'
    firstStripUrl = stripUrl % 'awakenings-pg-1'
    endOfLife = True

    def namer(self, imageUrl, pageUrl):
        filename = util.urlpathsplit(pageUrl)[-1]
        if filename.startswith('the-company-of-dragons'):
            filename = 'in-' + filename
        # Prepend chapter number to filename
        chapters = [
            'awakenings',
            'lifting-the-veil',
            'tears-of-blood',
            'on-the-lam',
            'shades-of-intrigue',
            'catfight',
            'out-of-control',
            'damage-control',
            'wolfs-clothing',
            'strange-dreams',
            'blood-bonds',
            'the-other-team',
            'get-ferrah',
            'the-price-of-power',
            'dogfight',
            'surfacing',
            'in-the-company-of-dragons',
            'filler',
        ]
        for chapter in chapters:
            if filename.startswith(chapter):
                filename = 'chapter-' + str(chapters.index(chapter) + 1) + '-' + filename
        return filename


class ShipInABottle(WordPressScraper):
    url = 'http://shipinbottle.pepsaga.com/'
    stripUrl = url + '?p=%s'
    firstStripUrl = stripUrl % '281'
    adult = True
    help = 'Index format: number'


class Shortpacked(ComicControlScraper):
    url = 'https://www.shortpacked.com/comic/'
    firstStripUrl = url + 'just-a-toy-store'


class ShotgunShuffle(WordPressSpliced):
    # Currently down, use archive.org in the meantime (08-2023)
    url = ('https://web.archive.org/web/20230131163842/'
        'https://shotgunshuffle.com/')
    firstStripUrl = url + 'comic/pilot/'


class SinFest(ParserScraper):
    END_HTML_TAG = compile(r'</html>')

    url = 'https://sinfest.xyz/'
    stripUrl = url + 'view.php?date=%s'
    firstStripUrl = stripUrl % '2000-01-17'
    imageSearch = '//img[contains(@src, "btphp/comics/")]'
    textSearch = imageSearch + '/@alt'
    prevSearch = '//a[d:class("prev")]'
    help = 'Index format: yyyy-mm-dd'

    # Remove HTML end tag confusing our parser
    def _parse_page(self, data):
        data = self.END_HTML_TAG.sub('', data)
        return super()._parse_page(data)


class SisterClaire(ComicControlScraper):
    url = 'https://www.sisterclaire.com/comic/'
    firstStripUrl = url + 'book-one'


class SixGunMage(ComicControlScraper):
    url = 'http://www.6gunmage.com/comic/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '6-gun-mage-kickoff'


class SixPackOfOtters(WordPressWebcomic):
    url = 'http://sixpackofotters.com/'
    stripUrl = url + 'pages/%s/'
    firstStripUrl = stripUrl % 'chapter-01-tandem'


class SkinDeep(WordPressWebcomic):
    url = 'https://www.skindeepcomic.com/'
    stripUrl = url + 'archive/%s/'
    firstStripUrl = stripUrl % 'issue-1-cover'
    imageSearch = '//div[d:class("webcomic-image")]//noscript/img'
    starter = bounceStarter
    namer = joinPathPartsNamer(pageparts=(-1,))


class SleeplessDomain(ComicControlScraper):
    url = 'http://www.sleeplessdomain.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'chapter-1-cover'
    starter = bounceStarter
    namer = joinPathPartsNamer(pageparts=(-1,))


class SlightlyDamned(ComicControlScraper):
    url = 'http://www.sdamned.com/'
    firstStripUrl = url + 'comic/prologue'
    starter = bounceStarter

    def namer(self, imageurl, pageurl):
        """Clean up mixed filename formats."""
        return 'SD' + util.urlpathsplit(pageurl)[-1]


class SluggyFreelance(ParserScraper):
    url = 'https://sluggy.com/'
    stripUrl = 'https://archives.sluggy.com/book.php?chapter=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//div[d:class("comic_content")]/img/@data-src'
    prevSearch = '//div[d:class("previous")]/a'
    latestSearch = '//a[d:class("archives_link")]'
    starter = indirectStarter
    multipleImagesPerStrip = True
    help = 'Index format: chapter'

    def namer(self, imageurl, pageurl):
        # Remove random noise from filename
        return util.urlpathsplit(imageurl)[-1].split('.', 1)[0]


class SMBC(ComicControlScraper):
    url = 'https://www.smbc-comics.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % '2002-09-05'
    imageSearch = ('//img[@id="cc-comic"]', '//div[@id="aftercomic"]/img')
    textSearch = '//img[@id="cc-comic"]/@title'
    multipleImagesPerStrip = True

    def namer(self, image_url, page_url):
        # Remove random noise from filename
        filename = util.urlpathsplit(image_url)[-1]
        if '-' in filename and len(filename.rsplit('-', 1)[-1]) > 12:
            filename = filename.rsplit('-', 1)[-1]
        elif len(filename) > 22 and filename[0] == '1':
            filename = filename[10:]
        return filename


class SnowFlame(WordPressScraper):
    url = ('https://web.archive.org/web/20160905071051/'
        'http://www.snowflamecomic.com/')
    stripUrl = url + '?comic=snowflame-%s-%s'
    firstStripUrl = stripUrl % ('01', '01')
    starter = bounceStarter
    endOfLife = True
    help = 'Index format: chapter-page'

    def getIndexStripUrl(self, index):
        return self.stripUrl % tuple(index.split('-'))

    def namer(self, image_url, page_url):
        filename = util.urlpathsplit(image_url)[-1]
        ro = compile(r'snowflame-([^-]+)-([^-]+)')
        mo = ro.search(page_url)
        chapter = mo.group(1)
        page = mo.group(2)
        return "%s-%s-%s" % (chapter, page, filename)


class SodiumEyes(WordPressScraper):
    url = 'https://web.archive.org/web/20200220041406/http://sodiumeyes.com/'
    starter = indirectStarter
    endOfLife = True


class SomethingPositive(_ParserScraper):
    url = 'https://www.somethingpositive.net/'
    stripUrl = url + 'sp%s.shtml'
    imageSearch = r'//img[re:test(@src, "/sp\d+")]'
    prevSearch = ('//a[contains(text(), "Previous")]',
        '//a[img[contains(@src, "previous")]]')
    multipleImagesPerStrip = True
    help = 'Index format: mmddyyyy'


class Sorcery101(WordPressWebcomic):
    baseUrl = 'https://kelmcdonald.com/sorcery-101/'
    stripUrl = baseUrl + '%s/'
    url = stripUrl % 'sorcery101-ch-01'
    firstStripUrl = url
    starter = indirectStarter
    help = 'Index format: stripname'


class SpaceFurries(ParserScraper):
    url = 'https://www.spacefurrs.org/'
    firstStripUrl = url
    multipleImagesPerStrip = True
    adult = True
    endOfLife = True

    def extract_image_urls(self, url, data):
        # Website requires JS, so build the list of image URLs manually
        imageurls = []
        current = int(self.match(data, '//input[@name="pagnum"]')[0].get('value'))
        for page in reversed(range(1, current + 1)):
            imageurls.append(self.url + 'comics/' + str(page) + '.jpg')
        return imageurls


class SpaceJunkArlia(ParserScraper):
    url = ('https://web.archive.org/web/20220121133701/'
        'http://spacejunkarlia.com/')
    stripUrl = url + '?strip_id=%s'
    firstStripUrl = stripUrl % '0'
    imageSearch = '//div[d:class("content")]/img'
    prevSearch = '//a[text()="<"]'
    endOfLife = True
    help = 'Index format: number'


class SpaceTrawler(_ParserScraper):
    url = 'https://www.baldwinpage.com/spacetrawler/'
    firstStripUrl = url + '2010/01/01/spacetrawler-4/'
    imageSearch = '//img[d:class("size-full")]'
    prevSearch = '//a[@rel="prev"]'


class Spamusement(_BasicScraper):
    url = 'http://spamusement.com/'
    rurl = escape(url)
    stripUrl = url + 'index.php/comics/view/%s'
    imageSearch = compile(r'<img src="(%sgfx/\d+\..+?)"' % rurl, IGNORECASE)
    prevSearch = compile(r'<a href="(%sindex.php/comics/view/.+?)">' % rurl,
                         IGNORECASE)
    latestSearch = prevSearch
    help = 'Index format: n (unpadded)'
    starter = indirectStarter


class SpareParts(_BasicScraper):
    baseUrl = 'http://www.sparepartscomics.com/'
    url = baseUrl + 'comics/?date=20080328'
    stripUrl = baseUrl + 'comics/index.php?date=%s'
    firstStripUrl = stripUrl % '20031022'
    imageSearch = compile(tagre("img", "src", r'(http://www\.sparepartscomics\.com/comics/[^"]+)'))
    prevSearch = compile(tagre("a", "href", r'(index\.php\?date=\d+)',
                               quote="'") + "Previous Comic")
    help = 'Index format: yyyymmdd'


class Spinnerette(ComicControlScraper):
    url = 'http://www.spinnyverse.com'


class SPQRBlues(WordPressScraper):
    url = 'http://spqrblues.com/IV/'


class SSDD(_ParserScraper):
    url = 'http://www.poisonedminds.com/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '19980927'
    imageSearch = ('//img[contains(@src, "/comics/")]',
                   '//source[contains(@src, "/video/")]')
    prevSearch = '//a[@rel="prev"]'
    multipleImagesPerStrip = True
    adult = True
    help = 'Index format: yyyymmdd'

    def shouldSkipUrl(self, url, data):
        # Skip news, flash animation, and non-comic pages.
        return url in (
            # News post
            self.stripUrl % '20060712',
            self.stripUrl % '20060719',
            self.stripUrl % '20071225',
            self.stripUrl % '20110321',
            self.stripUrl % '20110830',
            self.stripUrl % '20110929',
            self.stripUrl % '20180927',

            # Flash animation
            self.stripUrl % '20180401',
            self.stripUrl % '20170429',
            self.stripUrl % '20041203',

            # Comic missing
            self.stripUrl % '20070402',
            self.stripUrl % '20060413',
            self.stripUrl % '20060412',
            self.stripUrl % '20060202',
            self.stripUrl % '20051026',
            self.stripUrl % '20050805',
            self.stripUrl % '20050530',
            self.stripUrl % '20050526',
            self.stripUrl % '20050525',
            self.stripUrl % '20050524',
            self.stripUrl % '20050523',
            self.stripUrl % '20050504',
            self.stripUrl % '20040705',
            self.stripUrl % '20030418',
            self.stripUrl % '20030214',
        )


class StandStillStaySilent(_ParserScraper):
    baseUrl = 'http://sssscomic.com/'
    url = baseUrl + 'comic2.php'
    stripUrl = baseUrl + 'comic%s.php?page=%s'
    firstStripUrl = stripUrl % ('', '1')
    imageSearch = '//img[@class="comicnormal"]'
    prevSearch = '//a[./img[contains(@src, "nav_prev")]]'

    def namer(self, image_url, page_url):
        chapter = '2' if ('adv2_comicpages' in image_url) else '1'
        return '%s-%s' % (chapter, util.urlpathsplit(image_url)[-1].replace('page_', ''))


class StarCrossdDestiny(ParserScraper):
    baseUrl = ('https://web.archive.org/web/20190918132321/'
        'http://starcrossd.net/')
    url = baseUrl + 'comic.html'
    stripUrl = baseUrl + 'archives/%s.html'
    firstStripUrl = stripUrl % '00000001'
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[text()="prev"]'
    endOfLife = True
    help = 'Index format: nnnnnnnn'

    def namer(self, image_url, page_url):
        if image_url.find('ch1') == -1:
            # At first all images were stored in a strips/ directory but
            # that was changed with the introduction of book2
            image_url = sub('(?:strips)|(?:images)', 'book1', image_url)
        elif not image_url.find('strips') == -1:
            image_url = image_url.replace('strips/', '')
        directory, filename = util.urlpathsplit(image_url)[-2:]
        return directory + '-' + filename


class StarfireAgency(ParserScraper):
    url = 'https://poecatcomix.com/starfirecomic/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % 'sfa-issue-1-cover'
    imageSearch = '//img[@class="scale-with-grid wp-post-image"]'
    prevSearch = '//a[d:class("fixed-nav-prev")]'
    latestSearch = '//div[@class="post-title"]//a'
    starter = indirectStarter
    adult = True
    namer = joinPathPartsNamer(pageparts=(-1,))


class StarTrip(ComicControlScraper):
    url = 'https://www.startripcomic.com/'


class StationV3(_ParserScraper):
    url = 'http://www.stationv3.com/'
    stripUrl = url + 'd3/%s.html'
    firstStripUrl = stripUrl % '20170101'
    imageSearch = '//img[contains(@src,"/comics3/")]'
    prevSearch = '//a[img[contains(@src,"/previous2")]]'
    help = 'Index format: yyyymmdd'


class StickyDillyBuns(ComicControlScraper):
    url = 'https://pixietrixcomix.com/sticky-dilly-buns/'
    firstStripUrl = url + 'awesome-leading-man'
    endOfLife = True


class StreetFighter(ComicControlScraper):
    url = 'http://www.streetfightercomics.com'


class StringTheory(WordPressNavi):
    url = 'http://www.stringtheorycomic.com/'
    firstStripUrl = url + 'comics/chapterone/chapterone/'


class StrongFemaleProtagonist(ParserScraper):
    url = 'https://strongfemaleprotagonist.com/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % 'issue-1/page-0'
    imageSearch = '//article/p/img'
    prevSearch = '//a[d:class("page-nav__item--left")]'
    help = 'Index format: issue-?/page-??'

    def shouldSkipUrl(self, url, data):
        """Skip hiatus & non-comic pages."""
        return url in (
            self.stripUrl % 'guest-art/tuesday',
            self.stripUrl % 'guest-art/friday',
            self.stripUrl % 'guest-art/wednesday',
            self.stripUrl % 'issue-5/newspaper',
            self.stripUrl % 'issue-5/hiatus-1',
            self.stripUrl % 'issue-5/hiatus-2',
            self.stripUrl % 'issue-1/no-page',
        )


class StupidFox(ParserScraper):
    url = 'http://stupidfox.net/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % 'hello'
    imageSearch = '//div[d:class("comicmid")]//img'
    prevSearch = '//a[@accesskey="p"]'

    def namer(self, imageUrl, pageUrl):
        page = self.getPage(pageUrl)
        title = self.match(page, self.imageSearch + '/@title')[0].replace(' - ',
            '-').replace(' ', '-')
        return title + '.' + imageUrl.rsplit('.', 1)[-1]


class SuburbanJungle(_ParserScraper):
    url = 'http://suburbanjungleclassic.com/'
    stripUrl = url + '?p=%s'
    firstStripUrl = stripUrl % '10'
    imageSearch = '//div[@id="comic"]/img'
    prevSearch = '//div[@class="nav-previous"]/a'


class SuburbanJungleRoughHousing(WordPressScraper):
    url = 'http://roughhouse.suburbanjungle.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'rough-housing-issue-1-cover'


class Supercell(_ParserScraper):
    baseUrl = 'https://www.supercellcomic.com/'
    url = baseUrl + 'latest.html'
    stripUrl = baseUrl + 'pages/%s.html'
    firstStripUrl = stripUrl % '0001'
    imageSearch = '//img[@class="comicStretch"]'
    prevSearch = '//div[@class="comicnav"]/a[./img[contains(@src, "comnav_02")]]'


class SupernormalStep(ComicControlScraper):
    url = 'http://supernormalstep.com/'


class SurvivingTheWorld(_ParserScraper):
    url = 'http://survivingtheworld.net/'
    stripUrl = url + '%s.html'
    firstStripUrl = stripUrl % 'Lesson1'
    imageSearch = (
        '//div[@class="img"]/img',      # When there's one image per strip
        '//div[@class="img"]/p/img',    # When there's multiple images per strip
        '//td/img',                     # Special case for Lesson1296.html
    )
    prevSearch = (
        '//li[@class="previous"]/a',
        '//td/a',                       # Special case for Lesson1296.html
    )
    multipleImagesPerStrip = True
    help = 'Index format: name'


class SwordsAndSausages(_ParserScraper):
    url = 'https://www.tigerknight.com/ss'
    stripUrl = url + '/%s'
    firstStripUrl = stripUrl % '1-1'
    imageSearch = '//img[d:class("comic-image")]'
    prevSearch = '//a[./span[contains(text(), "Previous")]]'
    multipleImagesPerStrip = True


class SwordsComic(ParserScraper):
    url = 'https://swordscomic.com/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'cover'
    imageSearch = '//div[@class="page-image-wrapper"]//img'
    prevSearch = '//a[@class="navigation-button navigation-previous"]'
    help = 'Index format: Swordsnnn (unpadded)'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from __future__ import annotations

from re import compile, escape

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import (
    ComicControlScraper,
    WordPressNavi,
    WordPressScraper,
    WordPressWebcomic,
)


class CampComic(_ParserScraper):
    url = 'http://campcomic.com/comic/'
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '6'
    imageSearch = '//div[@id="comic"]/img'
    prevSearch = '//a[d:class("btnPrev")]'
    help = 'Index Format: number'


class CaptainSNES(_BasicScraper):
    url = 'http://www.captainsnes.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2001/07/10/the-mistake'
    imageSearch = compile(tagre("img", "src", r"(%scomics/[^']+)" % rurl,
                                quote="'"))
    prevSearch = compile(tagre("a", "href", r'(%s[^"]+)' % rurl) +
                         tagre("span", "class", "prev"))
    multipleImagesPerStrip = True
    help = 'Index format: yyyy/mm/dd/nnn-stripname'


class CarryOn(ParserScraper):
    url = 'https://www.hirezfox.com/km/co/co1024/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '20040701'
    imageSearch = '//div[d:class("strip")]/img'
    prevSearch = '//a[text()="Previous Day"]'
    multipleImagesPerStrip = True

    def namer(self, image_url, page_url):
        # Fix filenames of early comics
        filename = util.urlpathsplit(image_url)[-1]
        if filename[0].isdigit():
            filename = 'co' + filename
        return filename


class CarryOnAliceBlueAndTheGardensOfQ(CarryOn):
    name = 'CarryOn/AliceBlueAndTheGardensOfQ'
    url = 'https://www.hirezfox.com/km/abgq/abgq1024/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '20050401'
    endOfLife = True

    def namer(self, image_url, page_url):
        # Fix filenames
        return 'abgq' + util.urlpathsplit(image_url)[-1]


class CarryOnLegendOfAnneBunny(CarryOn):
    name = 'CarryOn/LegendOfAnneBunny'
    url = 'https://www.hirezfox.com/km/loab/loab1024/'
    stripUrl = url + 'd/%s.html'
    firstStripUrl = stripUrl % '20040701'
    endOfLife = True

    def namer(self, image_url, page_url):
        # Fix filenames of early comics
        filename = util.urlpathsplit(image_url)[-1]
        if filename[0].isdigit():
            filename = 'ab' + filename
        return filename


class CaseyAndAndy(_BasicScraper):
    url = 'http://www.galactanet.com/comic/'
    stripUrl = url + 'view.php?strip=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r'(Strip\d+\.gif)'))
    prevSearch = compile(tagre("a", "href", r'(view\.php\?strip=\d+)') +
                         tagre("img", "src", r'previous\.gif'))
    help = 'Index format: number'


class CassiopeiaQuinn(ComicControlScraper):
    url = 'https://www.cassiopeiaquinn.com/'
    firstStripUrl = url + 'comic/the-prize-cover'


class CasuallyKayla(ParserScraper):
    url = ('https://web.archive.org/web/20180323230312/'
        'https://casuallykayla.com/')
    stripUrl = url + '?p=%s'
    firstStripUrl = stripUrl % '89'
    imageSearch = '//div[@id="comic"]/img'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True
    help = 'Index format: nnn'


class Catalyst(_BasicScraper):
    baseUrl = "http://catalyst.spiderforest.com/"
    rurl = escape(baseUrl)
    url = baseUrl + "comic.php?comic_id=415"
    stripUrl = baseUrl + "comic.php?comic_id=%s"
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r'((?:%s)?comics/[^"]+)' % rurl))
    prevSearch = compile("<center>" +
                         tagre("a", "href",
                               r'(%scomic\.php\?comic_id=\d+)' % rurl))
    help = 'Index format: number'


class CatAndGirl(_ParserScraper):
    url = 'https://catandgirl.com/'
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[d:class("pager--prev")]'


class CatenaManor(ParserScraper):
    baseUrl = ('https://web.archive.org/web/20141027141116/'
        'http://catenamanor.com/')
    url = baseUrl + 'archives'
    stripUrl = baseUrl + '%s/'
    firstStripUrl = stripUrl % '2003/07'
    imageSearch = '//img[d:class("comicthumbnail")]'
    multipleImagesPerStrip = True
    endOfLife = True
    strips: list[str] = []

    def starter(self):
        # Retrieve archive links and select valid range
        archivePage = self.getPage(self.url)
        archiveStrips = self.match(archivePage, '//div[@id="archivepage"]//a')
        valid = False
        for link in archiveStrips:
            if self.stripUrl % '2012/01' in link.get('href'):
                valid = True
            elif self.stripUrl % '2003/06' in link.get('href'):
                valid = False
            if valid:
                self.strips.append(link.get('href'))
        return self.strips.pop(0)

    def getPrevUrl(self, url, data):
        return self.strips.pop(0)


class CatNine(WordPressScraper):
    url = 'https://classic.cat-nine.net/'
    firstStripUrl = url + 'comic/day-first'
    imageSearch = '//div[@id="comic"]//img/@data-src-img'
    endOfLife = True


class CatNineTakeTwo(WordPressScraper):
    name = 'CatNine/TakeTwo'
    url = 'https://cat-nine.net/'
    firstStripUrl = url + 'comic/episode-1/1-first-day-for-everything/'


class CatsAndCameras(WordPressScraper):
    url = 'https://catsncameras.com/cnc/'
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'cnc-begins'
    adult = True


class CatVersusHuman(_ParserScraper):
    url = 'http://www.catversushuman.com'
    imageSearch = '//div[@class="post-body entry-content"]//img'
    prevSearch = '//a[@id="Blog1_blog-pager-older-link"]'
    latestSearch = '//a[@rel="bookmark"]'
    starter = indirectStarter


class CavesAndCritters(WordPressWebcomic):
    url = 'https://cavesandcritters.com/?ao_confirm'
    stripUrl = 'https://cavesandcritters.com/cnc_webcomic/%s/'
    firstStripUrl = stripUrl % '01_000'
    adult = True


class Centralia2050(ParserScraper):
    url = 'https://centralia2050.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'ch1cover'
    imageSearch = '//div[@id="spliced-comic"]//img'
    prevSearch = '//a[@class="previous-comic"]'
    nextSearch = '//a[@class="next-comic"]'
    starter = bounceStarter

    def namer(self, imageUrl, pageUrl):
        page = util.urlpathsplit(pageUrl)[-1].replace('chapter', 'ch')
        if 'page-' in page and 'ch-' not in page:
            page = 'ch-1-' + page
        return page


class ChannelAte(WordPressNavi):
    url = 'http://www.channelate.com/'


class ChasingTheSunset(_BasicScraper):
    url = 'http://www.fantasycomic.com/'
    stripUrl = url + 'index.php?p=%s'
    firstStripUrl = stripUrl % 'c1'
    imageSearch = compile(r'(/cmsimg/.+?)".+?comic-img')
    prevSearch = compile(r'<a href="(.+?)" title="" ><img src="(images/eye-prev.png|images/cn-prev.png)"')
    help = 'Index format: n'


class Chester5000XYV(WordPressScraper):
    url = 'http://jessfink.com/Chester5000XYV/'
    stripUrl = url + '?p=%s'
    firstStripUrl = stripUrl % '34'
    prevSearch = '//a[@rel="prev"]'
    adult = True
    help = 'Index format: n (unpadded)'

    def link_modifier(self, fromurl, tourl):
        """Bugfix for link to blog"""
        if tourl == self.stripUrl % '714':
            return self.stripUrl % '710'
        return tourl


class Chisuji(WordPressScraper):
    url = 'http://www.chisuji.com/'
    stripUrl = url + '?p=%s'
    firstStripUrl = stripUrl % '266'
    prevSearch = '//div[@class="nav-previous"]/a'
    help = 'Index format: nnn'


class CigarroAndCerveja(_ParserScraper):
    url = 'http://www.cigarro.ca/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'reacquaintance'
    imageSearch = '//div[@id="comic"]//img'
    prevSearch = '//a[contains(text()," Prev")]'


class ClanOfTheCats(WordPressScraper):
    url = 'http://www.cotclassic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'coming-home-2'

    def link_modifier(self, fromurl, tourl):
        # Fix broken navigation link
        return tourl.replace('/2954/', '/2002-06-22/')


class ClanOfTheCatsReunion(WordPressScraper):
    name = 'ClanOfTheCats/Reunion'
    url = 'http://www.clanofthecats.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'cotc-reunion'


class Cloudscratcher(_ParserScraper):
    url = 'http://www.cloudscratcher.com/'
    stripUrl = url + 'comic.php?page=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "pages/")]'
    prevSearch = '//a[./img[@alt="Previous Page"]]'
    latestSearch = '//a[./img[@alt="Comic"]]'
    starter = indirectStarter


class CollegeCatastrophe(_ParserScraper):
    url = 'https://www.tigerknight.com/cc'
    stripUrl = url + '/%s'
    firstStripUrl = stripUrl % '2000-11-10'
    imageSearch = '//img[d:class("comic-image")]'
    prevSearch = '//a[./span[contains(text(), "Previous")]]'
    endOfLife = True
    multipleImagesPerStrip = True


class Comedity(_BasicScraper):
    url = 'http://www.comedity.com/'
    stripUrl = url + 'index.php?strip_id=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(r'<img src="(Comedity_files/.+?)"')
    prevSearch = compile(r'<a href="(/?index.php\?strip_id=\d+?)"> *<img alt=\"Prior Strip')
    help = 'Index format: n (no padding)'


class CommanderKitty(WordPressNavi):
    url = 'http://www.commanderkitty.com/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2009/01/03/good-to-be-back'
    endOfLife = True


class CommitStrip(ParserScraper):
    baseUrl = 'https://www.commitstrip.com/en/'
    url = baseUrl + '?setLocale=1'  # ensure the language cookie is set
    stripUrl = baseUrl + '%s/'
    firstStripUrl = stripUrl % '2012/02/22/interview'

    latestSearch = '//section//a'
    starter = indirectStarter
    namer = joinPathPartsNamer(pageparts=range(-4, 0), joinchar='-')
    imageSearch = '//article/div//img'
    prevSearch = '//span[@class="nav-previous"]/a'
    help = 'Index format: yyyy/mm/dd/strip-name'

    def link_modifier(self, fromurl, tourl):
        return tourl.replace('http:', 'https:')

    def shouldSkipUrl(self, url, data):
        """Skip pages without images."""
        return '2024/08/27/hello-world' in url


class CommitStripFr(CommitStrip):
    baseUrl = 'https://www.commitstrip.com/fr/'
    url = baseUrl + '?setLocale=1'  # ensure the language cookie is set
    stripUrl = baseUrl + '%s/'
    firstStripUrl = stripUrl % '2012/02/22/interview'
    lang = 'fr'


class CompanyY(_BasicScraper):
    url = 'http://company-y.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2009/08/14/coming-soon'
    imageSearch = compile(tagre("img", "src", r'(%scomics/[^"]+)' % rurl))
    prevSearch = compile(tagre("div", "class", r"nav-previous") +
                         tagre("a", "href", r'(%s[^"]+)' % rurl))
    help = 'Index format: yyyy/mm/dd/strip-name'


class Concession(_ParserScraper):
    url = 'http://concessioncomic.com/'
    stripUrl = url + 'index.php?pid=%s'
    firstStripUrl = stripUrl % '20060701'
    imageSearch = '//div[@id="comic"]/img[not(@class="preload")]'
    prevSearch = '//a[@class="nav-prev"]'
    adult = True
    endOfLife = True


class CorydonCafe(_ParserScraper):
    url = 'http://corydoncafe.com/'
    imageSearch = "//center[2]//img"
    prevSearch = '//a[@title="prev"]'
    multipleImagesPerStrip = True


class CourtingDisaster(WordPressScraper):
    url = 'https://web.archive.org/web/20201127150157/http://www.courting-disaster.com/'
    firstStripUrl = 'http://www.courting-disaster.com/comic/courting-disaster-17/'
    endOfLife = True


class CraftedFables(WordPressScraper):
    url = 'https://web.archive.org/web/20191126025641/http://www.caf-fiends.net/comicpress/'
    prevSearch = '//a[@rel="prev"]'
    endOfLife = True


class CrimsonDark(_BasicScraper):
    url = 'http://www.davidcsimon.com/crimsondark/'
    stripUrl = url + 'index.php?view=comic&strip_id=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(r'src="(.+?strips/.+?)"')
    prevSearch = compile(r'<a href=[\'"](/crimsondark/index\.php\?view=comic&amp;strip_id=\d+)[\'"]><img src=[\'"]themes/cdtheme/images/active_prev.png[\'"]')
    help = 'Index format: n (unpadded)'


class CrimsonFlag(_ParserScraper):
    url = 'http://crimsonflagcomic.com/'
    stripUrl = url + 'comic.php?comicID=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[@class="comicimage"]'
    prevSearch = '//a[contains(@class, "prev")]'


class CritterCoven(WordPressScraper):
    url = 'http://crittercoven.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'critter-coven'


class CrossTimeCafe(_ParserScraper):
    stripUrl = 'http://www.whiteponyproductions.com/ctc/%s.htm'
    url = stripUrl % 'present'
    firstStripUrl = stripUrl % 'ctc0001'
    imageSearch = '//img[not(contains(@src, "graphics/"))]'
    prevSearch = '//a[.//text()="Back"]'
    multipleImagesPerStrip = True
    endOfLife = True


class CSectionComics(WordPressScraper):
    url = 'https://www.csectioncomics.com/'
    firstStripUrl = url + 'comics/one-day-in-country'
    namer = joinPathPartsNamer(imageparts=range(-3, 0))
    multipleImagesPerStrip = True


class CtrlAltDel(WordPressWebcomic):
    url = 'https://cad-comic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'nice-melon'
    imageSearch = '//div[@class="comicpage"]//img[contains (@src,"/uploads/")]'
    prevSearch = '//a[@rel="prev"]'


class CucumberQuest(WordPressWebcomic):
    baseUrl = 'http://cucumber.gigidigi.com/'
    stripUrl = baseUrl + 'cq/%s/'
    firstStripUrl = stripUrl % 'page-1'
    url = firstStripUrl
    starter = indirectStarter
    help = 'Index format: stripname'


class Curtailed(WordPressScraper):
    url = 'https://www.curtailedcomic.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % '001-sneeze'

    def shouldSkipUrl(self, url, data):
        """Skip pages without images."""
        return 'comic/sitrep-1' in url or 'comic/be-right-back' in url


class Curvy(_ParserScraper):
    url = 'http://www.c.urvy.org/'
    stripUrl = url + '?date=%s'
    firstStripUrl = stripUrl % '20080329'
    imageSearch = '//div[@id="theActualComic"]//img'
    prevSearch = '//div[@class="aNavbar"]//p[2]/a'
    help = 'Index format: yyyymmdd'


class CutLoose(ParserScraper):
    url = 'https://www.cutloosecomic.com/'
    stripUrl = url + 'archive/comic/%s'
    firstStripUrl = stripUrl % '2016/02/02'
    imageSearch = '//img[@id="comic-container"]'
    prevSearch = '//a[@title="Previous Comic"]'
    nextSearch = '//a[@title="Next Comic"]'
    starter = bounceStarter
    adult = True

    def namer(self, image_url, page_url):
        postDate = '-'.join(util.urlpathsplit(page_url)[-3:])
        filename = util.urlpathsplit(image_url)[-1]
        return f'{postDate}_{filename}'


class CyanideAndHappiness(ParserScraper):
    url = 'https://explosm.net/'
    imageSearch = '//div[@id="comic"]//div[contains(@class,"ComicImage")]/span//img'
    prevSearch = '//div[@type="comic"]//a[*[local-name()="svg" and @rotate="180deg"]]'
    nextSearch = '//div[@type="comic"]//a[*[local-name()="svg" and @rotate="0deg"]]'
    starter = bounceStarter
    namer = joinPathPartsNamer(imageparts=range(-4, 0))


class CynWolf(_ParserScraper):
    url = 'https://cynwolf.net/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2008/because'
    imageSearch = '//section[contains(@class, "comic")]//img'
    prevSearch = '//a[text()="\u2190"]'
    multipleImagesPerStrip = True
    endOfLife = True

    def shouldSkipUrl(self, url, data):
        return '2016/the-end' in url  # video



# SPDX-License-Identifier: MIT
# Copyright (C) 2019-2022 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from ..scraper import ParserScraper


class ProjectFuture(ParserScraper):
    imageSearch = '//td[@class="tamid"]/img'
    prevSearch = '//a[./img[@alt="Previous"]]'

    def __init__(self, name, comic, first, last=None):
        if name == 'ProjectFuture':
            super(ProjectFuture, self).__init__(name)
        else:
            super(ProjectFuture, self).__init__('ProjectFuture/' + name)

        self.url = 'http://www.projectfuturecomic.com/' + comic + '.php'
        self.stripUrl = self.url + '?strip=%s'
        self.firstStripUrl = self.stripUrl % first

        if last:
            self.url = self.stripUrl
            self.endOfLife = True

    @classmethod
    def getmodules(cls):
        return (
            cls('AWalkInTheWoods', 'simeon', '1', last='12'),
            cls('BenjaminBuranAndTheArkOfUr', 'ben', '00', last='23'),
            cls('BookOfTenets', 'tenets', '01', last='45'),
            cls('CriticalMass', 'criticalmass', 'cover', last='26'),
            cls('DarkLordRising', 'darklord', '01-00', last='10-10'),
            cls('Emily', 'emily', '01-00', last='07-10'),
            cls('FishingTrip', 'fishing', '01-00', last='03-12'),
            cls('HeadsYouLose', 'heads', '00-01', last='07-12'),
            cls('InnocenceGuiltAndChange', 'ernst', '01-00'),
            cls('IPanther', 'panther', '00', last='20'),
            cls('NiallsStory', 'niall', '00'),
            cls('ProjectFuture', 'strip', '0', last='664'),
            cls('RedValentine', 'redvalentine', '1', last='6'),
            cls('ShortStories', 'shorts', '01-00'),
            cls('StrangeBedfellows', 'bedfellows', '1', last='6'),
            cls('TheAxemanCometh', 'axeman', '01-01', last='02-18'),
            cls('ToCatchADemon', 'daxxon', '01-00', last='03-14'),
            cls('TheDarkAngel', 'darkangel', 'cover', last='54'),
            cls('TheBountyProject', 'bounty', '00-00'),
            cls('TheEpsilonProject', 'epsilon', '00-01'),
            cls('TheHarvest', 'harvest', '01-00', last='08-26'),
            cls('TheSierraChronicles', 'sierra', '0', last='29'),
            cls('TheTuppenyMan', 'tuppenny', '00', last='16'),
            cls('TurningANewPage', 'azrael', '1', last='54'),
            cls('Xerian', 'xerian', '01-00'),
            cls('ZetaTakesATrip', 'zeta', '0-00'),
        )



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
"""
Automated comic downloader. Dosage traverses comic websites in
order to download each strip of the comic. The intended use is for
mirroring the strips locally for ease of viewing; redistribution of the
downloaded strips may violate copyright, and is not advisable unless you
have communicated with all of the relevant copyright holders, described
your intentions, and received permission to distribute.

The primary interface is the 'dosage' commandline script.
Comic modules for each comic are located in L{dosagelib.plugins}.
"""
import logging
from importlib import metadata

from . import logext  # noqa: F401 - needed here to preload logging extensions

logger = logging.getLogger(__name__)

AppName = 'dosage'
try:
    __version__ = metadata.version(AppName)  # PEP 396
except metadata.PackageNotFoundError:
    # package is not installed
    logger.warning('%s is not installed, no version available.'
        ' Use something like %r to fix this.', AppName, 'pip install -e .')
    __version__ = 'ERR.NOT.INSTALLED'



# SPDX-License-Identifier: MIT
# Copyright (C) 2004-2008 Tristan Seligmann and Jonathan Jacobs
# Copyright (C) 2012-2014 Bastian Kleineidam
# Copyright (C) 2015-2022 Tobias Gruetzmacher
# Copyright (C) 2019-2020 Daniel Ring
from .. import util
from ..helpers import bounceStarter
from ..scraper import ParserScraper


class xkcd(ParserScraper):
    url = 'https://xkcd.com/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//div[@id="comic"]//img'
    textSearch = imageSearch + '/@title'
    prevSearch = '//a[@rel="prev"]'
    nextSearch = '//a[@rel="next"]'
    starter = bounceStarter
    help = 'Index format: n (unpadded)'

    def namer(self, image_url, page_url):
        index = int(util.urlpathsplit(page_url)[-1])
        name = util.urlpathsplit(image_url)[-1]
        return '%04d-%s' % (index, name)

    def imageUrlModifier(self, url, data):
        if url and '/large/' in data:
            return url.replace(".png", "_large.png")
        return url



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: © 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: © 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: © 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: © 2019 Daniel Ring
from re import MULTILINE, compile

from .. import util
from ..helpers import bounceStarter, indirectStarter, joinPathPartsNamer
from ..scraper import ParserScraper, _BasicScraper, _ParserScraper
from ..util import tagre
from .common import WordPressNavi, WordPressScraper, WordPressWebcomic


class AbstruseGoose(ParserScraper):
    url = 'https://web.archive.org/web/20230930172141/https://abstrusegoose.com/'
    starter = bounceStarter
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "/strips/")]'
    textSearch = imageSearch + '/@title'
    textOptional = True
    prevSearch = '//a[contains(text(), "Previous")]'
    nextSearch = '//a[contains(text(), "Next")]'
    help = 'Index format: n (unpadded)'

    def namer(self, image_url, page_url):
        index = int(page_url.rsplit('/', 1)[1])
        name = util.urlpathsplit(image_url)[-1]
        return 'c%03d-%s' % (index, name)


class AbsurdNotions(_BasicScraper):
    baseUrl = 'http://www.absurdnotions.org/'
    url = baseUrl + 'page129.html'
    stripUrl = baseUrl + 'page%s.html'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre('img', 'src', r'(an[^"]+)'))
    multipleImagesPerStrip = True
    prevSearch = compile(tagre('a', 'href', r'([^"]+)') +
                         tagre('img', 'src', r'nprev\.gif'))
    help = 'Index format: n (unpadded)'


class Achewood(ParserScraper):
    baseUrl = 'https://achewood.com/'
    stripUrl = baseUrl + '%s/title.html'
    url = stripUrl % '2016/12/25'
    firstStripUrl = stripUrl % '2001/10/01'
    imageSearch = '//img[d:class("comicImage")]'
    prevSearch = '//a[d:class("comic_prev")]'
    namer = joinPathPartsNamer(pageparts=range(0, 3))
    help = 'Index format: yyyy/mm/dd'
    endOfLife = True


class AdventuresOfFifne(ParserScraper):
    stripUrl = 'http://fifine.purrsia.com/%s.html'
    url = stripUrl % 'COMICS'
    firstStripUrl = stripUrl % 'Fifine01'
    imageSearch = '//img[contains(@src, "jpg")]'
    prevSearch = '//a[text()="PREVIOUS"]'
    multipleImagesPerStrip = True
    endOfLife = True

    def namer(self, image_url, page_url):
        # Prepend chapter number to image filename
        filename = util.urlpathsplit(image_url)[-1]
        if filename[0] == 'p':
            filename = filename.replace('p', '1_p')
        filename = filename.replace('TIL', '2_TIL')
        filename = filename.replace('NS', '3_NS')
        filename = filename.replace('LG', '4_LG')
        filename = filename.replace('WM', '5_WM')
        return filename

    def getPrevUrl(self, url, data):
        # Fix broken navigation links
        if url == self.stripUrl % 'lg06':
            return self.stripUrl % 'lg05'
        return super(AdventuresOfFifne, self).getPrevUrl(url, data)


class AfterStrife(WordPressNavi):
    baseUrl = 'http://afterstrife.com/'
    stripUrl = baseUrl + '?p=%s'
    url = stripUrl % '262'
    firstStripUrl = stripUrl % '1'
    help = 'Index format: nnn'
    endOfLife = True


class AGirlAndHerFed(_ParserScraper):
    url = 'https://agirlandherfed.com/'
    stripUrl = url + '1.%s.html'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//div[@id="comic-image"]/img'
    prevSearch = '//div[@id="comic-nav"]/a[.//img[contains(@src, "back")]]'
    help = 'Index format: nnn'


class AhoiPolloi(_ParserScraper):
    url = 'https://ahoipolloi.blogger.de/'
    stripUrl = url + '?day=%s'
    firstStripUrl = stripUrl % '20060306'
    multipleImagesPerStrip = True
    lang = 'de'
    imageSearch = '//img[contains(@src, "/static/antville/ahoipolloi/")]'
    prevSearch = '//a[contains(@href, "/?day=")]'
    help = 'Index format: yyyymmdd'


class AirForceBlues(WordPressScraper):
    url = 'https://web.archive.org/web/20210102113825/http://farvatoons.com/'
    firstStripUrl = url + 'comic/in-texas-there-are-texans/'


class ALessonIsLearned(_BasicScraper):
    url = 'http://www.alessonislearned.com/'
    prevSearch = compile(tagre("a", "href", r"(index\.php\?comic=\d+)",
                               quote="'") + r"[^>]+previous")
    stripUrl = url + 'index.php?comic=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r"(cmx/lesson\d+\.[a-z]+)"))
    help = 'Index format: nnn'


class Alice(WordPressScraper):
    url = 'https://web.archive.org/web/20210115132313/http://www.alicecomics.com/'
    latestSearch = '//a[text()="Latest Alice!"]'
    starter = indirectStarter
    endOfLife = True


class AlienLovesPredator(ParserScraper):
    url = ('https://web.archive.org/web/20161207230717/'
        'http://alienlovespredator.com/')
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2004/10/12/unavoidable-delay'
    imageSearch = '//div[@id="comic"]//p//img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: yyyy/mm/dd/name'
    endOfLife = True


class AlienShores(WordPressScraper):
    url = 'http://alienshores.com/alienshores_band/'
    firstStripUrl = url + 'AScomic/updated-cover/'


class AllTheGrowingThings(WordPressScraper):
    url = ('https://web.archive.org/web/20160611212229/'
        'http://growingthings.typodmary.com/')
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % 'all-the-growing-things'
    endOfLife = True


class AlphaLuna(ParserScraper):
    url = 'https://alphalunacomic.net/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'issue-1-cover'
    imageSearch = '//main[@id="comic"]//img'
    prevSearch = '//a[@rel="prev"]'


class AlphaLunaSpanish(ParserScraper):
    name = 'AlphaLuna/Spanish'
    lang = 'es'
    url = 'https://alphalunacomic.net/spanish/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'issue-1-cover'
    imageSearch = '//main[@id="comic"]//img'
    prevSearch = '//a[@rel="prev"]'


class Altermeta(ParserScraper):
    url = 'http://altermeta.net/'
    stripUrl = url + 'archive.php?comic=%s'
    firstStripUrl = stripUrl % '0'
    imageSearch = '//img[contains(@src, "comics/")]'
    prevSearch = '//a[./img[contains(@src, "back")]]'
    nextSearch = '//a[./img[contains(@src, "forward")]]'
    starter = bounceStarter
    help = 'Index format: n (unpadded)'

    def namer(self, image_url, page_url):
        return page_url.rsplit('=', 1)[-1] + '_' + util.urlpathsplit(image_url)[-1]


class AltermetaOld(ParserScraper):
    url = Altermeta.url + 'oldarchive/index.php'
    stripUrl = Altermeta.url + 'oldarchive/archive.php?comic=%s'
    firstStripUrl = stripUrl % '0'
    imageSearch = '//img[contains(@src, "comics/")]'
    prevSearch = '//a[text()="Back"]'
    help = 'Index format: n (unpadded)'


class AmazingSuperPowers(WordPressNavi):
    url = 'https://www.amazingsuperpowers.com/'
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2007/09/heredity'
    imageSearch = '//div[d:class("comicpane")]/img'

    def shouldSkipUrl(self, url, data):
        """Skip pages without images."""
        return url in (
            # video
            self.stripUrl % '2013/05/orbital-deathray-kickstarter',
        )


class AmbersNoBrainers(_ParserScraper):
    baseUrl = 'https://foxyverse.com/'
    url = baseUrl + 'comics/'
    stripUrl = baseUrl + 'ambers-no-brainers-%s/'
    firstStripUrl = stripUrl % '1'
    imageSearch = '//img[contains(@src, "Page")]'
    latestSearch = '//a[contains(@href, "ambers-no-brainers")]'
    starter = indirectStarter

    def getPrevUrl(self, url, data):
        # Replace missing navigation links
        pageNum = int(url.rstrip('/').rsplit('-', 1)[-1])
        return self.stripUrl % str(pageNum - 1)


class Amya(WordPressScraper):
    url = 'http://www.amyachronicles.com/'


class Angband(ParserScraper):
    url = 'http://angband.calamarain.net/'
    stripUrl = url + '%s'
    imageSearch = '//img'
    multipleImagesPerStrip = True
    endOfLife = True

    def starter(self):
        page = self.getPage(self.url)
        self.pages = self.match(page, '//p/a[not(contains(@href, "cast"))]/@href')
        self.firstStripUrl = self.pages[0]
        return self.pages[-1]

    def getPrevUrl(self, url, data):
        return self.pages[self.pages.index(url) - 1]


class Annyseed(_ParserScraper):
    baseUrl = ('https://web.archive.org/web/20190511031451/'
        'http://www.mirrorwoodcomics.com/')
    stripUrl = baseUrl + 'Annyseed%s.htm'
    url = stripUrl % 'Latest'
    firstStripUrl = stripUrl % '000'
    imageSearch = '//div/img[contains(@src, "Annyseed")]'
    prevSearch = '//a[img[@name="Previousbtn"]]'
    endOfLife = True
    help = 'Index format: nnn'
    FIX_RE = compile(r'Annyseed/Finished%20For%20Print/')

    def imageUrlModifier(self, image_url, data):
        return self.FIX_RE.sub('', image_url)

    def link_modifier(self, fromurl, tourl):
        """Fix circular link."""
        if 'Annyseed150' in fromurl and 'Annyseed150' in tourl:
            return self.stripUrl % '149'
        return tourl


class AntiheroForHire(ParserScraper):
    stripUrl = 'https://www.giantrobot.club/antihero-for-hire/%s'
    firstStripUrl = stripUrl % '2016/6/8/entrance-vigil'
    url = firstStripUrl
    imageSearch = '//div[@class="image-wrapper"]//img[not(@class="thumb-image")]'
    multipleImagesPerStrip = True
    endOfLife = True

    def starter(self):
        # Build list of chapters for navigation
        page = self.getPage(self.url)
        self.chapters = self.match(page, '//ul[d:class("archive-group-list")]//a[d:class("archive-item-link")]/@href')
        return self.chapters[0]

    def getPrevUrl(self, url, data):
        # Retrieve previous chapter from list
        index = self.chapters.index(url) + 1
        return self.chapters[index] if index < len(self.chapters) else None


class AppleGeeks(_BasicScraper):
    url = 'http://www.applegeeks.com/'
    stripUrl = url + 'comics/viewcomic.php?issue=%s'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(tagre("img", "src", r'((?:/comics/)?issue\d+\.jpg)'))
    prevSearch = compile(r'<div class="caption">Previous Comic</div>\s*<p><a href="([^"]+)">', MULTILINE)
    allow_errors = (404,)
    help = 'Index format: n (unpadded)'


class ARedTailsDream(_BasicScraper):
    baseUrl = 'http://www.minnasundberg.fi/'
    stripUrl = baseUrl + 'comic/page%s.php'
    firstStripUrl = stripUrl % '00'
    url = baseUrl + 'comic/recent.php'
    imageSearch = compile(tagre('img', 'src', r'(chapter.+?/eng[^"]*)'))
    prevSearch = compile(tagre('a', 'href', r'(page\d+\.php)') +
                         tagre("img", "src", r'.*?aprev.*?'))
    help = 'Index format: nn'


class ArtificialIncident(WordPressWebcomic):
    url = 'https://www.artificialincident.com/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'issue-one-life-changing'


class AstronomyPOTD(ParserScraper):
    baseUrl = 'http://apod.nasa.gov/apod/'
    url = baseUrl + 'astropix.html'
    starter = bounceStarter
    stripUrl = baseUrl + 'ap%s.html'
    firstStripUrl = stripUrl % '061012'
    imageSearch = '//a/img'
    multipleImagesPerStrip = True
    prevSearch = '//a[text()="<"]'
    nextSearch = '//a[text()=">"]'
    help = 'Index format: yymmdd'

    def shouldSkipUrl(self, url, data):
        """Skip pages without images."""
        return self.match(data, '//iframe')  # videos

    def namer(self, image_url, page_url):
        return '%s-%s' % (page_url.split('/')[-1].split('.')[0][2:],
                          util.urlpathsplit(image_url)[-1])


class ATaleOfTails(WordPressScraper):
    url = 'http://www.feretta.net/'
    stripUrl = url + 'comic/%s/'
    firstStripUrl = stripUrl % 'a-tale-of-tails-1-0'
    adult = True


class Awaken(ParserScraper):
    url = "https://www.awakencomic.com/"
    stripUrl = url + 'comic/%s'
    firstStripUrl = stripUrl % 'comic-cover'
    imageSearch = '//div[@id="cc-comicbody"]//img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: chapter-n-page-n (unpadded)'


class AwkwardZombie(ParserScraper):
    url = 'https://www.awkwardzombie.com/'
    stripUrl = url + 'awkward-zombie/%s'
    firstStripUrl = stripUrl % 'coin-battle'
    imageSearch = '//div[@id="cc-comicbody"]//img'
    prevSearch = '//a[@rel="prev"]'
    help = 'Index format: variable :('


class AxeCop(WordPressScraper):
    url = 'http://axecop.com/comic/season-two/'



# SPDX-License-Identifier: MIT
# SPDX-FileCopyrightText: 2004 Tristan Seligmann and Jonathan Jacobs
# SPDX-FileCopyrightText: 2012 Bastian Kleineidam
# SPDX-FileCopyrightText: 2015 Tobias Gruetzmacher
# SPDX-FileCopyrightText: 2019 Daniel Ring
from re import compile, escape

from .. import util
from ..scraper import BasicScraper, ParserScraper
from ..util import tagre
from .common import WordPressNavi, WordPressScraper


class IAmArg(BasicScraper):
    url = 'http://iamarg.com/'
    rurl = escape(url)
    stripUrl = url + '%s/'
    firstStripUrl = stripUrl % '2011/05/08/05082011'
    imageSearch = compile(tagre("img", "src", r'(//iamarg.com/comics/\d+-\d+-\d+[^"]+)'))
    prevSearch = compile(tagre("a", "href", r'(%s\d+/\d+/\d+/[^"]+)' % rurl, after="prev"))
    help = 'Index format: yyyy/mm/dd/stripname'


class ICanBarelyDraw(BasicScraper):
    url = 'http://www.icanbarelydraw.com/comic/'
    rurl = escape(url)
    stripUrl = url + '%s'
    firstStripUrl = stripUrl % '39'
    imageSearch = compile(tagre("img", "src", r'(%scomics/\d+-\d+-\d+-[^"]+)' % rurl))
    prevSearch = compile(tagre("a", "href", r'(%s\d+)' % rurl))
    help = 'Index format: number'


class IDreamOfAJeanieBottle(WordPressScraper):
    url = 'http://jeaniebottle.com/'


class InternetWebcomic(WordPressNavi):
    url = 'http://www.internet-webcomic.com/'
    stripUrl = url + '?p=%s'
    firstStripUrl = stripUrl % '30'
    help = 'Index format: n'


class Inverloch(ParserScraper):
    stripUrl = 'https://www.seraph-inn.com/view.php?story=inverloch&page=%s'
    url = stripUrl % '763'
    firstStripUrl = stripUrl % '2'
    imageSearch = '//img[@class="page"]'
    prevSearch = '//p[@class="comic-nav"]/a[text()=" Previous"]'
    multipleImagesPerStrip = True
    endOfLife = True


class IrregularWebcomic(BasicScraper):
    url = 'http://www.irregularwebcomic.net/'
    stripUrl = url + '%s.html'
    firstStripUrl = stripUrl % '1'
    imageSearch = compile(r'<img .*src="(.*comics/.*(png|jpg|gif))".*>')
    prevSearch = compile(r'<a href="(/\d+\.html|/cgi-bin/comic\.pl\?comic=\d+)">Previous ')
    help = 'Index format: nnn'


class IslaAukate(ParserScraper):
    url = 'https://overlordcomic.com/archive/default/latest'
    stripUrl = 'https://overlordcomic.com/archive/default/pages/%s'
    firstStripUrl = stripUrl % '001'
    imageSearch = '//div[@id="comicpage"]/img'
    prevSearch = '//nav[@class="comicnav"]/a[text()="Prev"]'

    def namer(self, image_url, page_url):
        filename = util.urlpathsplit(image_url)[-1]
        return filename.rsplit('_', 1)[0]


class IslaAukateColor(ParserScraper):
    url = 'https://overlordcomic.com/archive/color/latest'
    stripUrl = 'https://overlordcomic.com/archive/color/pages/%s'
    firstStripUrl = stripUrl % '001'
    imageSearch = '//div[@id="comicpage"]/img'
    prevSearch = '//nav[@class="comicnav"]/a[text()="Prev"]'

    def namer(self, image_url, page_url):
        # Fix filenames of early comics
        filename = util.urlpathsplit(image_url)[-1]
        if filename[0].isdigit():
            filename = 'Aukate' + filename
        return filename.rsplit('_', 1)[0] + '.' + filename.rsplit('.', 1)[-1]


class ItsWalky(WordPressScraper):
    url = 'http://www.itswalky.com/'
